#import "Htag.h"
#import "structures.h"
#import <objc/HashTable.h>
#import <appkit/View.h>
#import <stdio.h>
#import <string.h>
#import <stdlib.h>
#import <appkit/Pasteboard.h>
#import <appkit/Panel.h>
#import <appkit/Listener.h>
#import <appkit/Speaker.h>
#import <mach/mach_init.h>
#import <mach/mach_interface.h>
#import "MiscList.h"
#import "TeXviewSwitchBoard.h"

extern TeXviewSwitchBoard *myTeXviewSwitchBoard ;
@implementation Htag
extern void         TPSpaintbox();
extern void         relativepage();
extern void         skippage();
extern void         freepages() ;
extern void         scanpages() ;
extern void         processbop() ;
extern void         dviseek() ;
extern integer      pagenum;
extern integer      curpos ;
extern integer      thispage ;
extern void         print_context() ;
extern shalfword    hh, vv;
extern float        screendpi ;
extern int          debugon ;
extern shalfword    hhoffset, vvoffset;
//extern id           htags;
extern void         dviGotoPage();
extern MiscList     *visited_tags;
extern int					visit_index;
extern void					writeconsole();
typedef struct {
    NXAtom              pat;
}                   char_vector;

void
WorkspaceOpenFile(char *fname)
{
    char                cmd[1000];

    sprintf(cmd, "open %s", fname);
    system(cmd);
}

- goToExternalTarget
{
    BOOL                done;
    id                  pasteboard;
    char_vector         mytypes[] = {NXAsciiPboardType, NULL};
    id                  mySpeaker;
    port_t              thePort;


/*Let's see if we can handle it ourselves*/
    if (strncmp(name, "file:", 5) == 0) {
	char                fname[1000];
	if (name[5] != '/') {
	    strcpy(fname, name + 5);
	    WorkspaceOpenFile(fname);
	    return self;
	}
	if (strncmp(name, "file:///", 8) == 0) {
	    strcpy(fname, name + 7);
	    WorkspaceOpenFile(fname);
	    return self;
	}
	if ((name[5] == '/') && (name[6] != '/')) {
	    strcpy(fname, name + 5);
	    WorkspaceOpenFile(fname);
	    return self;
	}
    }
/*No, pass it to OmniWeb*/
    thePort = NXPortFromName("OmniWeb", NULL);
    mySpeaker = [[Speaker alloc] init];
    [mySpeaker setSendPort:thePort];
    [mySpeaker free];		/* Frees the Speaker when it's no longer
				 * needed. */
    port_deallocate(task_self(), thePort);

    pasteboard = [Pasteboard new];
    [pasteboard declareTypes:(const char *const *)mytypes num:1 owner:self];
    [pasteboard writeType:NXAsciiPboardType
     data:(const char *)name
     length:(int)(strlen(name) + 1)];

    done = NXPerformService("OmniWeb/Open URL", pasteboard);
    if (!done)
	NXRunAlertPanel(NULL, "Could not connect to OmniWeb (not installed?) in order to access external url.",
			"OK", NULL, NULL);
    [pasteboard free];
    return self;
}

- setIsExternal:(int)myIsExternal
{
    isExternal = myIsExternal;
    return self;
}

- init
{
    name = 0;
    is_target = 0;
    return self;
}

- free
{
    if (name)
	free(name);
    return[super free];
}

- setIsTarget:(int)mytarget
{
    is_target = mytarget;
    return self;
}

- setMyName:(char *)myname
{
    if (name)
	free(name);
    name = (char *)malloc(strlen(myname) + 1);
    strcpy(name, myname);
    return self;
}

- setParams:(int)myfull_scanned :(int)myhh :(int)myvv :(int)mypage :(int)myihh :(int)myivv :(int)myipage :(float)myscreendpi;
{
    full_scanned = myfull_scanned;
    hh = myhh;
    vv = myvv;
    page = mypage;
    ihh = myihh;
    ivv = myivv;
    ipage = myipage;
    if ((ipage != page) || (hh < ihh)) {
	hh = ihh + 40;
	vv = ivv;
    }
    vv = ivv;
    scrdpi = myscreendpi;
    return self;
}
void
GenerateBox(int x1, int y1, int x2, int y2, double gray, double width)
{
    TPSpaintbox(x1, y1, x2, y1, x2, y2, x1, y2, gray, width);
}



- paintBox:(double)gray :(double)width
{
   if (screendpi != scrdpi) return self; //Don't paint the tags for another dpi
   GenerateBox(ihh + hhoffset, ivv + vvoffset, 
        hh + hhoffset, vv + vvoffset, gray, width);
    return self;
}

- (char *)getMyName
{
    return name;
}

- (int)getPage
{
    return ipage;
}

- (int)getiHH
{
    return ihh;
}

- (int)getiVV
{
    return ivv;
}

- (float)getScreenDpi
{
  return scrdpi ;
}

- printName;
{
		writeconsole(name);
		return self;
}

int                 ht_highlight = 0;
extern id           ht_highlight_tag;
int                 ht_last_page;
extern id           ht_last_clicked_tag;
extern int          seekpage();

- goToTarget:(int)iter:(int)cmd;
{
    char               *key;
    char               *name_dpi ;
    id                  value;
    NXHashState         state;
    id htags ;
    if (isExternal)
	return[self goToExternalTarget];
    htags = [myTeXviewSwitchBoard getCurrentHtags] ;
    name_dpi = (char *)malloc(strlen(name)+7) ;
    sprintf(name_dpi, "%sMD%.0f", name, screendpi) ;
    state = [htags initState];
    if (iter == 0) {
	ht_last_page = pagenum;
	ht_last_clicked_tag = self;
    }
    while ([htags nextState:&state key:(void *)&key value:(void *)&value]) {
#ifdef DEBUG
  if (debugon > 2 ) {
     printf("key is %s, but name_dpi is %s\n", key, name_dpi) ;
   }
#endif
	if ((strcmp(key, name_dpi) == 0)/*&& (screendpi==[value getScreenDpi])*/) {
	    char                pageStr[10];

	/* We found a target */
	    sprintf(pageStr, "%d",[value getPage]);
    /* Command-clicked so make a new slave or bring old to front */
      if (cmd) [myTeXviewSwitchBoard slaveToCurrent] ;
      ht_highlight = 3;
	    ht_highlight_tag = value;
      relativepage(1) ;
 	    dviGotoPage(pageStr);
      [myTeXviewSwitchBoard currentKeyAndFront] ;
      free(name_dpi) ;
	    return self;
	}
    }
    if (iter == 0) {
#ifdef DEBUG
  if (debugon > 1 ) {
  printf("\n before save in gototarget...\n");
  print_context() ;
  }
#endif
  [myTeXviewSwitchBoard saveCurrentContext] ;
	relativepage(-2);
  scanpages() ; // Scan pages but only processing specials, not fontdefs or
                // adding pages to linked list 
#ifdef DEBUG
  if (debugon > 1 ) {
  printf("Before restore...\n") ;
  print_context();
  }
#endif
  [myTeXviewSwitchBoard restoreCurrentContext] ;
#ifdef DEBUG
  if (debugon > 1 ) {
   printf("\n after restore in gototarget...\n");
   print_context() ;
  }
#endif
  dviseek(curpos) ; // reset the file pointer to the right place
  free(name_dpi) ;
	return[self goToTarget:(iter + 1):cmd];
    } else
  free(name_dpi) ;
#ifdef DEBUG
  if (debugon > 1 ) {
  printf("fell out the bottom\n") ;
  }
#endif
	return self;
}

- (int)checkIfHit:(int)x :(int)y :sheetview :(int)height :(int)cmd_click
{
    if (screendpi != scrdpi) return 0 ;
    if (x > hh + 1)
	return 0;
    if (x < ihh - 1)
	return 0;
    if (y > vv + 1)
	return 0;
    if (y < (ivv - height))
	return 0;
    if (!is_target) {
	[visited_tags insertObject:self at:(int) [visited_tags currentPosition]];
	[visited_tags setNextObject];
	// error("Added a tag to the list of visited tags");
	[visited_tags makeObjectsPerform:@selector(printName)];
	[self goToTarget:0:cmd_click];
  }  else {
/*Need to go back*/
	char                pageStr[10];

	sprintf(pageStr, "%d", ht_last_page);
	ht_last_page = pagenum;
	ht_highlight = 2;
	ht_highlight_tag = ht_last_clicked_tag;
	dviGotoPage(pageStr);
	[ht_last_clicked_tag showSelf:sheetview];
	ht_last_clicked_tag = self;
	[visited_tags setPreviousObject];
    }
    return 1;
}

- showSelf:sheetview;
{
    NXRect              myRect, tmpR;
    NXPoint             myPoint ;

    [sheetview getFrame:&tmpR];
    [sheetview getVisibleRect:&myRect];
  //  myRect.origin.x = hh + hhoffset - 60;
  //  myRect.origin.y = vv + vvoffset;
  //  myRect.size.width = hh - ihh + 2;
   // myRect.size.height = 20;
   // myRect.origin.y = tmpR.size.height - myRect.origin.y;
  // Center line containing tag in visible rectangle
    myPoint.y = tmpR.size.height - (vv + vvoffset + (myRect.size.height/2.0)) ;
    myPoint.x = ((tmpR.size.width/2.0) - (myRect.size.width/2.0)) ;
   // [sheetview scrollRectToVisible:&myRect];
    [sheetview scrollPoint:&myPoint] ;
    ht_highlight = 1;
    return self;
}

@end
