#   include	<stdio.h>
#   include	<stdlib.h>
#   include	<stddef.h>
#   include	<string.h>
#   include	<ctype.h>
#   include	<charnames.h>

#   include	"spelling.h"

/************************************************************************/
/*  Compare a word against a lemma in the dictionary.			*/
/*  This routine is nothing but a strcmp() variant.			*/
/************************************************************************/
#	define	ANNOTATIE( c )	( c == '*' || ( c >= '3' && c <= '9' ) )

int	nlSpelCmp( vorm1, regel )
const unsigned char *	vorm1;
const unsigned char *	regel;
    {
    const register unsigned char *	s1;
    const register unsigned char *	s2;
    int					c1;
    int					c2;
    int					cmp;

    s1= vorm1; s2= regel;

    c1= *s1; c2= *s2;
    while( ANNOTATIE( c1 ) )
	{ c1= *(++s1);	}
    while( ANNOTATIE( c2 ) )
	{ c2= *(++s2);	}

    while( ( c1= *s1 ) && c1 != '#' )
	{
	if  ( ISO_isupper( c1 ) )	{ c1= ISO_tolower( c1 );	}
	if  ( ISO_isupper( c2 ) )	{ c2= ISO_tolower( c2 );	}

	if  ( c1 != c2 )
	    { break;	}

	c1= *(++s1); c2= *(++s2);
	while( ANNOTATIE( c1 ) )
	    { c1= *(++s1);	}
	while( ANNOTATIE( c2 ) )
	    { c2= *(++s2);	}
	}

    while( ( c2= *s2 ) && ANNOTATIE( c2 ) )
	{ s2++;	}

    if  ( c1 == '#' )	{ c1= '\0';	}
    if  ( c2 == '#' )	{ c2= '\0';	}

    cmp= c1 - c2;

    return cmp;
    }

/************************************************************************/
/*  Transform an entry in a lemma to a normal string.			*/
/*  This routine is nothing but a strcpy() variant.			*/
/*  Vertaling is naar NeXT's Extended character set.			*/
/************************************************************************/
unsigned char *	nlSpelCpy( unsigned char * vorm, const unsigned char * s )
    {
    unsigned char *	buf= vorm;
    int			trema= 0;

    while( *s )
	{
	while( *s && ! ANNOTATIE( *s ) )
	    {
	    *(buf++)= *(s++);
	    if  ( trema )
		{
		if  ( buf[-1] == 'a' )	{ buf[-1]= ISO_adieresis;	}
		if  ( buf[-1] == 'e' )	{ buf[-1]= ISO_edieresis;	}
		if  ( buf[-1] == 'i' )	{ buf[-1]= ISO_idieresis;	}
		if  ( buf[-1] == 'o' )	{ buf[-1]= ISO_odieresis;	}
		if  ( buf[-1] == 'u' )	{ buf[-1]= ISO_udieresis;	}
		trema= 0;
		}
	    }
	while( *s && ANNOTATIE( *s ) )
	    {
	    if  ( *s == '3' )	/* nje */
		{
		if  ( buf[-1] == 'n' )	{ buf[-1]= ISO_ntilde;	}
		}
	    if  ( *s == '4' )	/*  trema */
		{ trema= 1;	}
	    if  ( *s == '5' )	/*  umlaut  */
		{
		if  ( buf[-1] == 'a' )	{ buf[-1]= ISO_adieresis;	}
		if  ( buf[-1] == 'e' )	{ buf[-1]= ISO_edieresis;	}
		if  ( buf[-1] == 'i' )	{ buf[-1]= ISO_idieresis;	}
		if  ( buf[-1] == 'o' )	{ buf[-1]= ISO_odieresis;	}
		if  ( buf[-1] == 'u' )	{ buf[-1]= ISO_udieresis;	}
		}
	    if  ( *s == '6' )	/*  grave  */
		{
		if  ( buf[-1] == 'a' )	{ buf[-1]= ISO_agrave;	}
		if  ( buf[-1] == 'e' )	{ buf[-1]= ISO_egrave;	}
		if  ( buf[-1] == 'i' )	{ buf[-1]= ISO_igrave;	}
		if  ( buf[-1] == 'o' )	{ buf[-1]= ISO_ograve;	}
		if  ( buf[-1] == 'u' )	{ buf[-1]= ISO_ugrave;	}
		}
	    if  ( *s == '7' )	/*  aigu  */
		{
		if  ( buf[-1] == 'a' )	{ buf[-1]= ISO_aacute;	}
		if  ( buf[-1] == 'e' )	{ buf[-1]= ISO_eacute;	}
		if  ( buf[-1] == 'i' )	{ buf[-1]= ISO_iacute;	}
		if  ( buf[-1] == 'o' )	{ buf[-1]= ISO_oacute;	}
		if  ( buf[-1] == 'u' )	{ buf[-1]= ISO_uacute;	}
		}
	    if  ( *s == '8' )	/*  circumflex  */
		{
		if  ( buf[-1] == 'a' )	{ buf[-1]= ISO_acircumflex;	}
		if  ( buf[-1] == 'e' )	{ buf[-1]= ISO_ecircumflex;	}
		if  ( buf[-1] == 'i' )	{ buf[-1]= ISO_icircumflex;	}
		if  ( buf[-1] == 'o' )	{ buf[-1]= ISO_ocircumflex;	}
		if  ( buf[-1] == 'u' )	{ buf[-1]= ISO_ucircumflex;	}
		}
	    if  ( *s == '9' )	/*  cedilla  */
		{
		if  ( buf[-1] == 'c' )	{ buf[-1]= ISO_ccedilla;	}
		if  ( buf[-1] == 'C' )	{ buf[-1]= ISO_Ccedilla;	}
		}
	    s++;
	    }
	}
    *buf= '\0';

    return vorm;
    }

/************************************************************************/
/*  Heeft een woordenboek vorm zekere uitgang?				*/
/************************************************************************/
static int nlKlemToon( beklemtoond, vorm, vorml, uitgangl )
int			beklemtoond;
const unsigned char *	vorm;
int			vorml;
int			uitgangl;
    {
    if  ( beklemtoond == 0 )
	{ return 1;	}
    if  ( beklemtoond > 0 && vorm[vorml- uitgangl- 1] == '*' )
	{ return 1;	}
    if  ( beklemtoond < 0 && vorm[vorml- uitgangl- 1] != '*' )
	{ return 1;	}

    return 0;
    }

int nlUitgang(	const unsigned char * vorm, int vorml,
		const unsigned char * uitgang, int uitgangl,
		int beklemtoond )
    {
    if  ( uitgangl >= vorml )
	{ return 0;	}
    if  ( strcmp( vorm+ vorml- uitgangl, uitgang ) )
	{ return 0;	}

    if  ( nlKlinker( uitgang[0] )		&&
	  vorml > uitgangl+ 1			&&
	  nlKlinker( vorm[vorml- uitgangl- 1] )	)
	{
	if  ( vorml < uitgangl+ 3					||
	      vorm[vorml- uitgangl- 1] != 'i'				||
	      ( strncmp( vorm+ vorml- uitgangl- 3, "aai", 3 )	&&
	        strncmp( vorm+ vorml- uitgangl- 3, "ooi", 3 )	&&
	        strncmp( vorm+ vorml- uitgangl- 3, "oei", 3 )	)	)
	    {
	    if  ( vorml < uitgangl+ 2					||
		  ( strncmp( vorm+ vorml- uitgangl- 2, "ui", 2 ) &&
		    strncmp( vorm+ vorml- uitgangl- 2, "au", 2 ) &&
		    strncmp( vorm+ vorml- uitgangl- 2, "ou", 2 ) )	)
		{ return 0;	}
	    }
	}

    return nlKlemToon( beklemtoond, vorm, vorml, uitgangl );
    }

int	nlKlinker( int c )
    {
    return	c == 'a'		||
		c == 'e'		||
		c == 'i'		||
		c == 'o'		||
		c == 'u'		||
		c == 'y'		||
		c == ISO_adieresis	||
		c == ISO_edieresis	||
		c == ISO_idieresis	||
		c == ISO_odieresis	||
		c == ISO_udieresis	||
		c == ISO_acircumflex	||
		c == ISO_ecircumflex	||
		c == ISO_icircumflex	||
		c == ISO_ocircumflex	||
		c == ISO_ucircumflex	||
		c == ISO_agrave		||
		c == ISO_egrave		||
		c == ISO_igrave		||
		c == ISO_ograve		||
		c == ISO_ugrave		||
		c == ISO_aacute		||
		c == ISO_eacute		||
		c == ISO_iacute		||
		c == ISO_oacute		||
		c == ISO_uacute;
    }
