/************************************************************************/
/*  Lemma als te vinden in woordenlijst.				*/
/*  Zie de bij horende 'Leesmij' file.					*/
/************************************************************************/
#   include	<stdio.h>


typedef struct	lm_s
    {
    unsigned char *	lm_vorm1;
    unsigned char *	lm_vorm2;
    unsigned char *	lm_vorm3;
    unsigned char *	lm_vorm4;

    char		lm_soort;
    char		lm_geslacht;
    char		lm_wederkerend;
    char		lm_samen;
    char		lm_hoeveelheids;
    char		lm_intervoc_d;
    } Lemma;

					/********************************/
					/*  Woordsoorten.		*/
					/*  Oorspronkelijk.		*/
					/********************************/
#   define	TEL	5		/*  Telwoord.			*/
#   define	ADJ	10		/*  Bijvoegelijk naamwoord.	*/
#   define	ADV_C	21		/*  Verbuigbaar bijwoord.	*/
#   define	ADV	22		/*  Onverbuigbaar bijwoord.	*/
#   define	VGW	30		/*  Voegwoord.			*/
#   define	VRZ	40		/*  Voorzetsel			*/
#   define	LDW	50		/*  Lidwoord.			*/
#   define	INT	60		/*  Tussenwerpsel.		*/
#   define	PRO	70		/*  Voornaamwoord.		*/
#   define	SUB	83		/*  Zelfstandig naamwoord.	*/
#   define	SUB_D	84		/*  Zelfstandig verkleinwoord.	*/
#   define	SUB_P	85		/*  Plurale Tantum.		*/
#   define	VER	90		/*  Werkwoord.			*/
					/********************************/
					/*  Toegevoegd.			*/
					/********************************/
#   define	NAAM	81		/*  Persoonsnaam.		*/
#   define	TOPO	82		/*  Geografische naam.		*/
#   define	SUB_C	86		/*  Collectivum: heeft geen	*/
					/*  verbuigingen.		*/
#   define	VORM	99		/*  Moeilijk te classificeren	*/
					/********************************/

					/********************************/
					/*  Geslachten.			*/
					/********************************/
#   define	SEX_MV	0		/*  mv.				*/
#   define	SEX_M	1		/*  m.				*/
#   define	SEX_V	2		/*  v.				*/
#   define	SEX_O	3		/*  o.				*/
#   define	SEX_VM	4		/*  v. (m.)			*/
#   define	SEX_M_V	5		/*  m. -v.			*/
#   define	SEX_MeO	6		/*  m. en o.			*/
#   define	SEX_MeV	7		/*  m. en v.			*/
#   define	SEX_VeM	8		/*  v. en m.			*/
#   define	SEX_OeV	9		/*  o. en v.			*/
#   define	SEX_VeO	10		/*  v. en o.			*/
#   define	SEX_OeM	11		/*  o. en m.			*/
#   define	SEX_MoO	12		/*  m. of o.			*/
#   define	SEX_MoV	13		/*  m. of v.			*/
#   define	SEX_VoO	14		/*  v. of o.			*/
					/********************************/

/************************************************************************/
/*  Derivatie.								*/
/************************************************************************/
typedef struct	dv_s
    {
    char	dv_soort;		/*  Woordsoort.			*/

    char	dv_persoon;		/*  ik,jij,hij,wij,jullie,zij	*/
    char	dv_klein;		/*  Verkleinwoord.		*/
    char	dv_wijstrap;		/*  De wijs van een ww vorm.	*/
    char	dv_tijd;		/*  Werkwooordstijd.		*/
    char	dv_verbogen;		/*  bijvoegelijk naamw. +e	*/
    int		dv_dertype;		/*  Type in <morpho.h>		*/
    } Deriv;

#   define	dv_wijs	dv_wijstrap
#   define	dv_trap	dv_wijstrap

/************************************************************************/
/*  Constructors.							*/
/************************************************************************/
typedef struct
    {
    Deriv	der_deriv;
    int		(*der_MaakErEen)( Lemma *, unsigned char * );
    } Derivator;

					/********************************/
					/*  Personen			*/
					/********************************/
#   define	SING_1	1		/*  1e persoon enkelvoud.	*/
#   define	SING_2	2		/*  2e persoon enkelvoud.	*/
#   define	SING_3	3		/*  3e persoon enkelvoud.	*/
#   define	PLUR_1	11		/*  1e persoon meervoud.	*/
#   define	PLUR_2	12		/*  2e persoon meervoud.	*/
#   define	PLUR_3	13		/*  3e persoon meervoud.	*/
					/********************************/


					/********************************/
					/*  De 'wijzen' van werkwoorden.*/
					/********************************/
#   define	INF	 	21	/*  Infinitief.			*/
#   define	IND	 	22	/*  Indicatief.			*/
#   define	CONJUNCTIEF	23	/*  Conjunctief.		*/
#   define	IMPERATIEF	24	/*  Imperatief.			*/
#   define	TEGDEEL		25	/*  Tegenwoordig deelwoord.	*/
#   define	VERLDEEL	26	/*  Verleden deelwoord.		*/
					/********************************/

					/********************************/
					/*  De trappen van vergelijking	*/
					/********************************/
#   define	POS	41		/*  Stellende trap		*/
#   define	COMP	42		/*  Vergelijkende trap		*/
#   define	SUPER	43		/*  Overtreffende trap.		*/
					/********************************/

					/********************************/
					/*  De tijden van werkwoorden.	*/
					/********************************/
#   define	PRES	61		/*  Tegenwoordige tijd.		*/
#   define	IMPER	62		/*  Verleden tijd.		*/
					/********************************/

/************************************************************************/
/*  Declaraties. Diverse routines.					*/
/************************************************************************/
extern int		nlSpelCmp( const unsigned char *,
				const unsigned char * );
extern unsigned char *	nlSpelCpy( unsigned char *,
				const unsigned char * );
extern int		nlMaak( void * ind, Lemma * plemma, int,
			    void * indvreemd );
extern int		nlUitgang( const unsigned char *,
				int, const unsigned char *, int, int );
extern int		nlKlinker( int );
extern void		nlDrukDeriv( unsigned char *, Lemma *, Deriv * );
extern void		nlDrukLemma( Lemma * );
extern void		nlSplitLemma( char *, Lemma * );
extern int		nlPut( void * ind, const unsigned char * vorm,
			    int type );

extern int nlRoepProc(	FILE *		f,
			long		filesize,
			unsigned char *	woord,
			void *		door,
			int		(*proc)( Lemma *, void * )	);

/************************************************************************/
/*  Derivaties voor substantieven.					*/
/************************************************************************/
extern int		nlSubPlur( Lemma * plemma, unsigned char * buf );
extern int		nlSubDimSing1( Lemma * plemma, unsigned char * buf );
extern int		nlSubDimSing2( Lemma * plemma, unsigned char * buf );
extern int		nlSubDimPlur1( Lemma * plemma, unsigned char * buf );
extern int		nlSubDimPlur2( Lemma * plemma, unsigned char * buf );

/************************************************************************/
/*  Derivaties voor adjectieven.					*/
/************************************************************************/
extern int		nlAdjBuig( Lemma * plemma, unsigned char * buf );
extern int		nlAdjBuigVorm( const unsigned char * vorm1,
				    unsigned char * buf );
extern int		nlAdjBuigPlur( Lemma * plemma, unsigned char * buf );
extern int		nlAdjComp( Lemma * plemma, unsigned char * buf );
extern int		nlAdjSuper( Lemma * plemma, unsigned char * buf );
extern int		nlAdjCompE( Lemma * plemma, unsigned char * buf );
extern int		nlAdjCompEN( Lemma * plemma, unsigned char * buf );
extern int		nlAdjSuperE( Lemma * plemma, unsigned char * buf );
extern int		nlAdjSuperEN( Lemma * plemma, unsigned char * buf );
extern int		nlAdjAllerSuper( Lemma * plemma, unsigned char * buf );
extern int		nlAdjAllerSuperE( Lemma * plemma, unsigned char * buf );
extern int		nlAdjSubst( Lemma * plemma, unsigned char * buf );
extern int		nlAdjCompSubst( Lemma * plemma, unsigned char * buf );

/************************************************************************/
/*  Derivaties voor voornaamwoorden.					*/
/************************************************************************/
extern int		nlProBuig1( Lemma * plemma, unsigned char * buf );
extern int		nlProBuig2( Lemma * plemma, unsigned char * buf );

/************************************************************************/
/*  Derivaties voor bijwoorden.					*/
/************************************************************************/
extern int		nlAdvComp( Lemma * plemma, unsigned char * buf );
extern int		nlAdvSuper( Lemma * plemma, unsigned char * buf );

/************************************************************************/
/*  Derivaties voor Telwoorden.					*/
/************************************************************************/
extern int		nlTelBuigS( Lemma * plemma, unsigned char * buf );
extern int		nlTelBuigP( Lemma * plemma, unsigned char * buf );
extern int		nlTelGener( void * ind, void * indvreemd );

/************************************************************************/
/*  Derivaties voor werkwoorden.					*/
/************************************************************************/
extern int		nlVerPresS1( Lemma * plemma, unsigned char * buf );
extern int		nlVerPresS2( Lemma * plemma, unsigned char * buf );
extern int		nlVerPresS2nl( Lemma * plemma, unsigned char * buf );
extern int		nlVerPresS3( Lemma * plemma, unsigned char * buf );
extern int		nlVerPresP123( Lemma * plemma, unsigned char * buf );
extern int		nlVerPresS1s( Lemma * plemma, unsigned char * buf );
extern int		nlVerPresS2s( Lemma * plemma, unsigned char * buf );
extern int		nlVerPresS3s( Lemma * plemma, unsigned char * buf );
extern int		nlVerPresP123s( Lemma * plemma, unsigned char * buf );
extern int		nlVerImperS123( Lemma * plemma, unsigned char * buf );
extern int		nlVerImperP123( Lemma * plemma, unsigned char * buf );
extern int		nlVerImperS123s( Lemma * plemma, unsigned char * buf );
extern int		nlVerImperP123s( Lemma * plemma, unsigned char * buf );
extern int		nlVerPartPas( Lemma * plemma, unsigned char * buf );
extern int		nlVerPartPasE( Lemma * plemma, unsigned char * buf );
extern int		nlVerPartPasEN( Lemma * plemma, unsigned char * buf );
extern int		nlVerPartPres( Lemma * plemma, unsigned char * buf );
extern int		nlVerPartPresE( Lemma * plemma, unsigned char * buf );
extern int		nlVerPartPresEN( Lemma * plemma, unsigned char * buf );
extern int		nlVerCompPref( Lemma * plemma, unsigned char * buf );
extern int		nlVerGeStam( Lemma * plemma, unsigned char * buf );
