#   include	<ind.h>
#   include	"indlocal.h"
#   include	<charnames.h>
#   include	<debugon.h>

/************************************************************************/
/*  See whether a word is accepted.					*/
/*  As this is a utility routine for spelling checking, the following	*/
/*  attempts are made to accept it when it is not quite acceptable:	*/
/************************************************************************/
int indWRDget( ind, pWhatWasShifted, word, asPrefix )
IND *			ind;
int *			pWhatWasShifted;
const unsigned char *	word;
int			asPrefix;
    {
    static unsigned char *	copy;

    int				l;
    const unsigned char *	r;
    unsigned char *		s;
    int				accepted;
    int				tn;

    tn= indINDget( &accepted, ind, -1, word );
    if  ( tn >= 0 && ( accepted || asPrefix ) )
	{ *pWhatWasShifted= INDhASIS; return 0;	}

    l= strlen( (char *)word );
    copy= realloc( copy, l+ 2 );

    if  ( ! copy )
	{ LDEB(copy); return -1;	}

    strcpy( (char *)copy, (char *)word );

    if  ( ISO_isupper( word[0] ) )
	{
	if  ( word[0] == 'I' && word[1] == 'J' )
	    {
	    copy[0]= 'i'; copy[1]= 'j';
	    if  ( indINDget( &accepted, ind, -1, copy ) >= 0	&&
		  ( accepted || asPrefix )			)
		{ *pWhatWasShifted= INDhIJUP; return 0;	}
	    }
	else{
	    if  ( ! ISO_isupper( word[1] ) )
		{
		copy[0]= ISO_tolower( word[0] );
		if  ( indINDget( &accepted, ind, -1, copy ) >= 0	&&
		      ( accepted || asPrefix )			)
		    { *pWhatWasShifted= INDhFIRSTUP; return 0;	}
		}
	    }

	/************************************************/
	/*  Shift down.					*/
	/************************************************/
	r= word; s= copy;
	while( *r && ISO_isupper( *r ) )
	    {
	    *s= ISO_tolower(*r);
	    s++; r++;
	    }

	/************************************************/
	/*  Not completely in upper case		*/
	/************************************************/
	if  ( *r )
	    { return -1; }

	/************************************************/
	/*  Try shift down.				*/
	/************************************************/
	if  ( indINDget( &accepted, ind, -1, copy ) >= 0	&&
	      ( accepted || asPrefix )			)
	    { *pWhatWasShifted= INDhALLUP; return 0;	}

	/************************************************/
	/*  Try tail shifted down.			*/
	/************************************************/
	if  ( word[0] == 'I' && word[1] == 'J' )
	    {
	    copy[0]= word[0]; copy[1]= word[1];
	    if  ( indINDget( &accepted, ind, -1, copy ) >= 0	&&
		  ( accepted || asPrefix )			)
		{ *pWhatWasShifted= INDhIJTAILUP; return 0;	}
	    }
	else{
	    copy[0]= word[0];
	    if  ( indINDget( &accepted, ind, -1, copy ) >= 0	&&
		  ( accepted || asPrefix )			)
		{ *pWhatWasShifted= INDhTAILUP; return 0;	}
	    }
	}

    return -1;
    }

/************************************************************************/
/*  Generate guesses for a word.					*/
/************************************************************************/
int	indWRDguess(	IND *				ind,
			const unsigned char *		word,
			void *				something,
			const GuessSubstitution *	typos,
			int				count,
			int				(*fun)
				( void *, const unsigned char *, int, int ) )
    {
    int				l;
    static unsigned char *	copy;

    const unsigned char *	r;
    unsigned char *		s;

    l= strlen( (char *)word );
    copy= realloc( copy, l+ 2 );
    if  ( ! copy )
	{ LDEB(copy); return -1;	}
    strcpy( (char *)copy, (char *)word );

    if  ( ISO_isupper( word[0] ) )
	{
	if  ( word[0] == 'I' && word[1] == 'J' )
	    {
	    copy[0]= 'i'; copy[1]= 'j';
	    indINDguess( ind, copy, something, INDhIJUP,
						typos, count, fun );
	    }
	else{
	    if  ( ! ISO_isupper( word[1] ) )
		{
		copy[0]= ISO_tolower( word[0] );
		indINDguess( ind, copy, something, INDhFIRSTUP,
							typos, count, fun );
		}
	    }

	/************************************************/
	/*  Shift down.					*/
	/************************************************/
	r= word; s= copy;
	while( *r && ISO_isupper( *r ) )
	    {
	    *s= ISO_tolower(*r);
	    s++; r++;
	    }

	/************************************************/
	/*  Not completely in upper case		*/
	/************************************************/
	if  ( *r )
	    { return 0; }

	/************************************************/
	/*  Try shift down.				*/
	/************************************************/
	indINDguess( ind, copy, something, INDhALLUP, typos, count, fun );

	/************************************************/
	/*  Try tail shifted down.			*/
	/************************************************/
	if  ( word[0] == 'I' && word[1] == 'J' )
	    {
	    copy[0]= word[0]; copy[1]= word[1];
	    indINDguess( ind, copy, something, INDhIJTAILUP,
						    typos, count, fun );
	    }
	else{
	    copy[0]= word[0];
	    indINDguess( ind, copy, something, INDhTAILUP, typos, count, fun );
	    }
	}
    else{
	indINDguess( ind, word, something, INDhASIS, typos, count, fun );
	}

    return 0;
    }

int	indShiftWord( copy, word, how )
unsigned char *		copy;
const unsigned char *	word;
int			how;
    {
    int	i;

    strcpy( copy, word );

    switch( how )
	{
	case	INDhASIS:
	    break;
	case	INDhFIRSTUP:
	    if  ( ISO_islower( copy[0] ) )
		{ copy[0]= ISO_toupper( copy[0] ); }
	    break;
	case	INDhIJUP:
	    if  ( copy[0] == 'i' && copy[1] == 'j' )
		{ copy[0]= 'I'; copy[1]= 'J';	}
	    break;
	case	INDhALLUP:
	    i= 0;
	    while( copy[i] )
		{
		if  ( ISO_islower( copy[i] ) )
		    { copy[i]= ISO_toupper( copy[i] ); }
		i++;
		}
	    break;
	case	INDhTAILUP:
	    if  ( ! ISO_isupper( copy[0] ) )
		{ return -1;	}
	    i= 1;
	    while( copy[i] )
		{
		if  ( ISO_islower( copy[i] ) )
		    { copy[i]= ISO_toupper( copy[i] ); }
		i++;
		}
	    break;
	case	INDhIJTAILUP:
	    if  ( copy[0] != 'I' || copy[1] != 'J' )
		{ return -1;	}
	    i= 2;
	    while( copy[i] )
		{
		if  ( ISO_islower( copy[i] ) )
		    { copy[i]= ISO_toupper( copy[i] ); }
		i++;
		}
	    break;
	default:
	    return -1;
	}

    return 0;
    }
