/************************************************************************/
/*  									*/
/*  Non interactive spelling checker.					*/
/*  									*/
/*  Usage:								*/
/*	mspell <flags> <index> <files>					*/
/*  Where:								*/
/*	<flags>		Control the operation the following exist:	*/
/*			-guess	Guesses are given along with refused	*/
/*				words.					*/
/*			-next	Input is supposed to be NeXT encoding.	*/
/*			-dos437	Input is supposed to be in MS-DOS code 	*/
/*				page 437 encoding, rather than ISO.	*/
/*	<index>		Is a valid 'ind' spelling index.		*/
/*	<files>		Is a list of files to be checked.		*/
/*  									*/
/*  Rejected words are printed on standard output, one below the other.	*/
/*  If guesses are enabled, these are printed each on separate lines	*/
/*  below the rejected word. Each guess is preceded by a tab.		*/
/*  									*/
/************************************************************************/

#   import	<stdio.h>
#   import	<ctype.h>
#   import	<stdlib.h>
#   import	<string.h>
#   import	<debugoff.h>

#   include	<charnames.h>
#   include	<dosiso.h>
#   include	<nextiso.h>
#   include	<ind.h>

/* #   define	LOG(x)	x */
#   define	LOG(x)

#   define	BUL	250
static int	_c_;
#   define spelWordChar( c )	(isalnum(_c_=(c))||!isascii(_c_))

static int spelLagBehind;
static int spelPos;
static unsigned char *	spelConversion;

# define SPELMAX 8
static char spelCollected[SPELMAX];

static int spelGetc( f )
FILE *	f;
    {
    int	c;

    if  ( spelLagBehind > 0 )
	{ spelLagBehind--; return spelCollected[spelPos++]; }
    spelPos= 0;

    c= getc( f );

    if  (  c > 0 && spelConversion )
	{ c= spelConversion[ c ]; }

    if  ( c == '-' )
	{
	spelLagBehind= 1; spelPos= 1;
	spelCollected[0]= c;
	spelCollected[1]= c= getc( f );
	if  ( c == '\r' )
	    { spelCollected[1]= c= getc( f );	}
	if  ( c == '\n' )
	    {
	    c= getc( f );
	    spelLagBehind= 0;
	    return c;
	    }
	else{ spelPos= 1; return spelCollected[0];	}
	}

    if  ( c == '\r' )
	{
	spelCollected[0]= c= getc( f );
	if  ( c != '\n' )
	    { LDEB(c);	}
	}

    if  ( c == '\n' )
	{ c= ' ';	}

    return c;
    }

static int spellGuess(	void *			voidigl,
			const unsigned char *	word,
			int			score,
			int			how		)
    {
    unsigned char	copy[100];

    if  ( indShiftWord( copy, word, how ) )
	{ return 0;	}

    if  ( indAddGuess( (IndGuessList *)voidigl, copy, score ) )
	{ SLDEB(copy,score); return -1;	}

    return 0;
    }

static int spelCheckWord(	const unsigned char *	word,
				void *			ind,
				int			asPrefix )
    {
    int			ignoredHow;

    if  ( ! asPrefix && isdigit( word[0] ) )
	{ return 0;	}

    if  ( ! indGetWord( &ignoredHow, ind, word, asPrefix ) )
	{ return 0;	}

    return -1;
    }

static int mspel( FILE * infle, void * ind, int guess )
    {
    int			c;
    int			rval= 0;
    int			count;

    int			position= -1;
    PossibleWord *	possibilities= (PossibleWord *)0;

    spelLagBehind= 0;
    spelPos= 0;

    for (;;)
	{
	while( ( c= spelGetc( infle ) ) != EOF )
	    {
	    position++;
	    if  ( spelWordChar( c ) )
		{ break;	}
	    }

	if  ( c == EOF )
	    {
	    indFreePossibilities( possibilities );
	    possibilities= (PossibleWord *)0;
	    return rval;
	    }

	indAddCharacterToPossibilities( possibilities, c );

	possibilities= indNewPossibility( position, possibilities, c );
	if  ( ! possibilities )
	    { LDEB(possibilities); exit(1);	}

	while( ( c= spelGetc( infle ) ) != EOF )
	    {
	    position++;
	    if  ( ! spelWordChar( c ) )
		{ break;	}

	    indAddCharacterToPossibilities( possibilities, c );
	    }

	count= indCountPossibilities( possibilities, position, spelCheckWord,
					    (void *)ind, c );

	/*
	indLogPossibilities( possibilities );
	LDEB(count);
	*/

	if  ( count == 0 )
	    {
	    PossibleWord *	maxpw;

	    maxpw= indMaximalPossibility( possibilities );

	    printf( "%s\n", (char *)maxpw->pwForm );
	    rval= 1;

	    if  ( guess )
		{
		static		IndGuessList		igl;
		int		limit;
		int		i;

		indCleanGuesses( &igl );

		indGuessWord( ind, maxpw->pwForm, (void *)&igl,
				(GuessSubstitution *)0, 0, spellGuess );

		indSortGuesses( &igl );

		if  ( igl.iglGuessCount > 6 )
		    {
		    limit= ( igl.iglGuesses[0].igsScore+
			2* igl.iglGuesses[igl.iglGuessCount-1].igsScore )/ 3;
		    }
		else{ limit= 0;	}

		for ( i= 0; i < igl.iglGuessCount; i++ )
		    {
		    if  ( igl.iglGuesses[i].igsScore < limit )
			{ /*LLDEB(igl.iglGuesses[i].igsScore,limit);*/ break; }

		    printf( "\t%s\n", (char *)igl.iglGuesses[i].igsWord );
		    }
		}
	    }

	possibilities= indRejectPossibilities( possibilities );
	indAddCharacterToPossibilities( possibilities, c );
	}

    return rval;
    }

int main( int argc, char ** argv )
    {
    int			rval= 0;
    FILE *		infle;
    void *		ind;
    int			guess= 0;

    while( argc >= 3 && argv[1][0] == '-' )
	{
	if  ( ! strcmp( argv[1], "-dos437" ) )
	    { argv++; argc--; spelConversion= Dos437ToIso; continue;	}
	if  ( ! strcmp( argv[1], "-next" ) )
	    { argv++; argc--; spelConversion= NextToIso; continue;	}
	if  ( ! strcmp( argv[1], "-guess" ) )
	    { argv++; argc--; guess= 1; continue;	}
	break;
	}

    if  ( argc < 2 )
	{
	fprintf( stderr, "argc = %d, not 3\n", argc );
	return 1;
	}

    ind= indRead( argv[1], 1 );
    if  ( ! ind )
	{
	fprintf( stderr, "Cannot open '%s'\n", argv[1] );
	return 1;
	}

    if  ( argc == 2 )
	{
	if  ( mspel( stdin, ind, guess ) )
	    { rval= 1;	}
	}
    else{
	while( argc >= 3 )
	    {
	    infle= fopen( argv[2], "r" );
	    if  ( ! infle )
		{
		fprintf( stderr, "Cannot open '%s'\n", argv[2] );
		rval= 1; argv++; argc--; continue;
		}

	    if  ( mspel( infle, ind, guess ) )
		{ rval= 1;	}

	    fclose( infle );

	    argc--; argv++;
	    }
	}

    indFree( ind );

    return rval;
    }
