#   import	<stdio.h>
#   import	<ctype.h>
#   import	<stdlib.h>
#   import	<string.h>

#   import	<appkit/Application.h>
#   import	<appkit/NXSpellServer.h>
#   import	<appkit/readOnlyTextStream.h>
#   import	<appkit/NXCType.h>

#   include	<ind.h>
#   include	<nextspel.h>
#   include	<debugon.h>

extern unsigned char		IsoToNext[256];
extern unsigned char		NextToIso[256];
static unsigned char		spelCopy[400];

/************************************************************************/
/*  A spelling checker for the NeXT work station. It is no more than a	*/
/*  front end to the 'ind' routines.					*/
/*  NOTE the following:							*/
/*	1)	It opens the index $LANGUAGE.ind in the bundle the	*/
/*		executable was started from. This will do for the	*/
/*		moment.							*/
/*	2)	Handles are in the code for learing and forgetting	*/
/*		words. An internal administartion is kept, and a text	*/
/*		file $LANGUAGE.changes is kept in			*/
/*		$HOME/.NeXT/dictionaries.				*/
/*	3)	Contrary to the 'NXSpellServer' document from NeXT,	*/
/*		installation is in $HOME/Library/Services.		*/
/*	4)	From experimentation, it is clear that a spell server	*/
/*		should itself look for the beginning of a word in the	*/
/*		text that is offered to it. Otherwise it cannot skip to	*/
/*		the next word if the user decides to ignore its opinion	*/
/*		on ortography. Fortunately, a possible word on position	*/
/*		0 in the text is found through the wrap around. This	*/
/*		saves us from keeping a state where we should not.	*/
/************************************************************************/

/************************************************************************/
/*  FAKE getc() on NeXT streams: nxSpelGetc(), nxSpelFgetc()		*/
/*	The ReadOnlyTextStream Protocol is so unfriendly, that it was	*/
/*	worth to fake the UNIX stdio package on them. No FILE struct is	*/
/*	is used but , buffer, count and pointer are local variables	*/
/*	and pointers to them are passed to the macro or routine.	*/
/*	For this simple purpose, fgetc() and _filbuf() were		*/
/*	indistinguishable and merged into one routine			*/
/************************************************************************/
#   define	BUL	500
static int nxSpelFgetc( char * buf, id textStream, char ** pptr, int * pcnt );
#   define nxSpelGetc(b,t,p,c)	\
				( (-- *(c))>=0	?		\
				    *((*(p))++)	:		\
				    nxSpelFgetc(b,t,p,c)	)

typedef struct	SpelGuessInfo
    {
    id			sgiServer;
    IndGuessList *	sgiList;
    } SpelGuessInfo;

/************************************************************************/

@interface indSpeller:	Object
    {
    void *			staticInd;
    void *			learntInd;
    void *			forgotInd;
    int				typoCount;
    const GuessSubstitution *	typos;
    }

-   initArgv0:		(const char *)			argv0
    language:		(const char *)			language
    typos:		(const GuessSubstitution *)	x_typos
    typoCount:		(int)				x_count;

-   (BOOL)spellServer:		(NXSpellServer *)	sender
    findMisspelledWord:		(int *)			start
    length:			(int *)			length
    inLanguage:			(char *)		language
    inTextStream:	(id <NXReadOnlyTextStream>)	textStream
    startingAt:			(int)			startPosition
    wordCount:			(int *)			number
    countOnly:			(BOOL)			flag;

-   (void)spellServer:		(NXSpellServer *)	sender
    suggestGuessesForWord:	(const char *)		word
    inLanguage:			(const char *)		language;

-   (void)spellServer:		(NXSpellServer *)	sender
    didLearnWord:		(const char *)		word
    inLanguage:			(const char *)		language;

-   (void)spellServer:		(NXSpellServer *)	sender
    didForgetWord:		(const char *)		word
    inLanguage:			(const char *)		language;

-   (void)						free;

@end

static int	spelGuess( void * voidserver, const unsigned char * word, int score, int how );
static int	toPrivateDict( const char * prefix, const char * word,
				const char * language );
static int	spelMoveWord( void * fromInd, void * toInd,
				unsigned char * word );

@implementation	indSpeller:	Object

/************************************************************************/
/*  Open the changes file.						*/
/************************************************************************/
static FILE *	spelChangesFile(	const char *	language,
					const char *	mode		)
    {
    const char *	homedir= NXHomeDirectory();
    char		fullName[1000];
    FILE *		rval;

    if  ( ! homedir )
	{ return (FILE *)0;	}

    sprintf( fullName, "%s/.NeXT/dictionaries/%s.changes",
	    homedir, language );

    rval= fopen( fullName, mode );

    return rval;
    }

-   initArgv0:		(const char *)			argv0
    language:		(const char *)			language
    typos:		(const GuessSubstitution *)	x_typos
    typoCount:		(int)				x_count
    {
    char *		indexName;
    int			l1, l2;
    FILE *		f;

    [ super init ];

    typos= x_typos;
    typoCount= x_count;

    staticInd= (void *)0;
    learntInd= (void *)0;
    forgotInd= (void *)0;

    indexName= strrchr( argv0, '/' );
    if  ( ! indexName )
	{ [ self free ]; return (id)0;	}
    l1= indexName- argv0;
    l2= strlen( language );
    indexName= malloc( l1+ l2+ 5 );
    if  ( ! indexName )
	{ [ self free ]; return (id)0;	}
    sprintf( indexName, "%*.*s/%s.ind", l1, l1, argv0, language );

    /* DEB(printf( "INDEX '%s'\n", indexName )); */

    staticInd= indRead( indexName, 1 );
    if  ( ! staticInd )
	{ SXDEB(indexName,staticInd); [ self free ]; return (id)0;	}

    free( indexName );

    learntInd= indMake();
    if  ( ! learntInd )
	{ [ self free ]; return (id)0;	}

    forgotInd= indMake();
    if  ( ! forgotInd )
	{ [ self free ]; return (id)0;	}

    f= spelChangesFile( language, "r" );

#   define SPELBUFL	300
    if  ( f )
	{
	unsigned char	buf[SPELBUFL+ 1];


	while( fgets( (char *)buf, SPELBUFL, f ) == (char *)buf )
	    {
	    buf[strlen( buf )- 1]= '\0';

	    switch( buf[0] )
		{
		case 'F':
		    (void) spelMoveWord(	learntInd,
						forgotInd,
						buf+ 2 );
		    break;
		case 'L':
		    (void) spelMoveWord(	forgotInd,
						learntInd,
						buf+ 2 );
		    break;
		default:
		    DEB(printf("STRANGE CHANGES LINE '%s'\n", buf ));
		    break;
		}
	    }
	}

    fclose( f );

    return self;
    }

-   (void) free
    {
    if  ( staticInd )
	{ indFree( staticInd );	}
    if  ( learntInd )
	{ indFree( learntInd );	}
    if  ( forgotInd )
	{ indFree( forgotInd );	}

    [ super free ];
    }

/************************************************************************/
/*									*/
/*  Struct to pass information through indCountPossibilities().		*/
/*									*/
/*  Give an opinion on a word.						*/
/*  Called through indCountPossibilities().				*/
/*									*/
/*  1)  Explicitly forbidden?						*/
/*  2)  In dictionary?							*/
/*  3)  Learnt?								*/
/*  4)  Also give NextStep a chance.					*/
/*									*/
/*  5)  Now the word is to be rejected. Consider the possibility of a	*/
/*	fixed combination such as:					*/
/*	'drie-eenheid'	(Dutch)						*/
/*	'ten behoeve'	(Dutch)						*/
/*	'a priori'	(Latin, Phylosophy)				*/
/*	'Buenos Aires'	(Brazil)					*/
/*	'Addis Abeba'	(Ethiopia)					*/
/*									*/
/************************************************************************/
typedef struct SpelCheckContext
    {
    NXSpellServer *	sccSender;
    indSpeller *	sccSpeller;
    } SpelCheckContext;

static int spelCheckWord(	const unsigned char *	word,
				void *			vscc,
				int			asPrefix )
    {
    SpelCheckContext *	scc=	(SpelCheckContext *)vscc;

    int				accepted;
    int				ignoredHow;
    const unsigned char *	r= word;
    unsigned char *		s= spelCopy;

    if  ( ! asPrefix && isdigit( word[0] ) )
	{ return 0;	}

    while( *r )
	{ *(s++)= NextToIso[*(r++)];	}
    *s= '\0';

    if  ( ! asPrefix )
	{
	/*  1  */
	if  ( indGet( &accepted, scc->sccSpeller->forgotInd,
						spelCopy ) >= 0	&&
	      accepted						)
	    { return -1;	}
	}

    /*  2  */
    if  ( ! indGetWord( &ignoredHow, scc->sccSpeller->staticInd,
						    spelCopy, asPrefix ) )
	{ return 0;	}

    /*  3  */
    if  ( indGet( &accepted, scc->sccSpeller->learntInd,
						    spelCopy ) >= 0	&&
	  ( accepted || asPrefix )					)
	{ return 0; }

    if  ( ! asPrefix )
	{
	/*  4  */
	if  ( [ scc->sccSender	isInUserDictionary:	word
				caseSensitive:		YES	] )
	    { return 0;	}
	}

    return -1;
    }

/************************************************************************/
/*  The actual checker.							*/
/*  1)  If the insertion point was in the middle of a word. First skip	*/
/*	to the end.							*/
/*  2)  Skip space &c upto the beginning of a 'word'.			*/
/*  3)  Here starts a new word. Start to collect it.			*/
/************************************************************************/
#   define	spelWordChar( c )	(NXIsAlNum(c))

-   (BOOL)spellServer:		(NXSpellServer *)	sender
    findMisspelledWord:		(int *)			start
    length:			(int *)			length
    inLanguage:			(char *)		language
    inTextStream:	(id <NXReadOnlyTextStream>)	textStream
    startingAt:			(int)			startPosition
    wordCount:			(int *)			number
    countOnly:			(BOOL)			flag
    {
    /********************/
    /*  A la getc()	*/
    /********************/
    char	buf[BUL+1];
    int		cnt= 0;
    char *	ptr= buf;
    /********************/

    int			c;

    int			words= 0;
    int			position= startPosition;

    PossibleWord *	possibilities= (PossibleWord *)0;

    /*  1  */
    if  ( startPosition > 0 )
	{
	while( ( c= nxSpelGetc( buf, textStream, &ptr, &cnt ) ) != EOF )
	    {
	    position++;
	    if  ( ! spelWordChar( c ) )
		{ break;	}
	    }
	}

    for (;;)
	{
	/*  2  */
	while( ( c= nxSpelGetc( buf, textStream, &ptr, &cnt ) ) != EOF )
	    {
	    position++;
	    if  ( spelWordChar( c ) )
		{ break;	}
	    }

	if  ( c == EOF )
	    {
	    indFreePossibilities( possibilities );
	    *start= position; *length= 0; *number= 0; return NO;
	    }


	/*  3  */
	if  ( flag == NO )
	    {
	    indAddCharacterToPossibilities( possibilities, c );

	    possibilities= indNewPossibility( position- 1, possibilities, c );
	    if  ( ! possibilities )
		{ LDEB(possibilities); exit(1);	}
	    }

	while( ( c= nxSpelGetc( buf, textStream, &ptr, &cnt ) ) != EOF )
	    {
	    position++;
	    if  ( ! spelWordChar( c ) )
		{ break;	}

	    if  ( flag == NO )
		{ indAddCharacterToPossibilities( possibilities, c );	}
	    }

	/*
	DEB(indLogPossibilities(possibilities));
	*/

	if  ( flag == NO )
	    {
	    SpelCheckContext	scc;
	    int			count;

	    scc.sccSender=	sender;
	    scc.sccSpeller=	self;

	    count= indCountPossibilities( possibilities, position,
					    spelCheckWord, (void *)&scc, c );

	    if  ( count == 0 )
		{
		PossibleWord *	maxpw;

		maxpw= indMaximalPossibility( possibilities );

		*start= maxpw->pwStartPosition;
		*length= maxpw->pwInsertionPoint;
		*number= words;

		indFreePossibilities( possibilities );

		return YES;
		}

	    if  ( flag == NO )
		{
		possibilities= indRejectPossibilities( possibilities );
		indAddCharacterToPossibilities( possibilities, c );
		}
	    }

	words++;
	}
    }

/************************************************************************/
/*  Remember a guess for a word.					*/
/************************************************************************/
static int spelGuess(	void *			voidsgi,
			const unsigned char *	word,
			int			score,
			int			how )
    {
    int			accepted;
    int			i= 0;
    IndGuessList *	igl= ((SpelGuessInfo *)voidsgi)->sgiList;
    id			server= ((SpelGuessInfo *)voidsgi)->sgiServer;
    unsigned char	copy[100];

    /*
    DEB(printf("spelGuess(,word=\"%s\",score=%d,how=%d)\n", word, score, how ));
    */

    if  ( indShiftWord( copy, word, how ) )
	{ /*SLDEB(word,how);*/ return 0;	}

    while( copy[i] )
	{ copy[i]= IsoToNext[copy[i]]; i++;	}

    /************************************/
    /*  Forbidden?			*/
    /************************************/
    {
    indSpeller *	speller= [ server delegate ];

    if  ( indGet( &accepted, speller->forgotInd, copy ) >= 0	&&
	  accepted						)
	{ return 0;	}
    }

    if  ( indAddGuess( igl, copy, score ) )
	{ /*SLDEB(copy,score);*/ return -1;	}

    return 0;
    }

/************************************************************************/
/*  Make a series of guesses for a word.				*/
/************************************************************************/
-   (void)spellServer:		(id)			sender
    suggestGuessesForWord:	(const char *)		wrd
    inLanguage:			(const char *)		language
    {
    const unsigned char *	r= (const unsigned char *)wrd;
    unsigned char *		s= spelCopy;
    int				i;
    int				limit;
    SpelGuessInfo		sgi;

    static IndGuessList		igl;

    sgi.sgiServer= sender;
    sgi.sgiList= &igl;

    /*
    DEB(printf( "NEXT GUESSES FOR '%s' IN LANGUAGE '%s'\n", wrd, language ));
    */

    while( *r )
	{ *(s++)= NextToIso[*(r++)];	}
    *s= '\0';

    indCleanGuesses( &igl );

    indGuess    ( learntInd, spelCopy, (void *)&sgi, INDhASIS,
						typos, typoCount, spelGuess );
    indGuessWord( staticInd, spelCopy, (void *)&sgi,
						typos, typoCount, spelGuess );

    indSortGuesses( &igl );

    if  ( igl.iglGuessCount > 6 )
	{
	limit= ( igl.iglGuesses[0].igsScore+
		    2* igl.iglGuesses[igl.iglGuessCount-1].igsScore )/ 3;
	}
    else{ limit= 0;	}

    for ( i= 0; i < igl.iglGuessCount; i++ )
	{
	if  ( igl.iglGuesses[i].igsScore < limit )
	    { /*LLDEB(igl.iglGuesses[i].igsScore,limit);*/ break;	}

	[ sender addGuess:	igl.iglGuesses[i].igsWord ];
	}

    return;
    }

/************************************************************************/
/*  Store a word in the private dictionary.				*/
/************************************************************************/
static int	toPrivateDict( prefix, word, language )
const char *	prefix;
const char *	word;
const char *	language;
    {
    FILE *		f= spelChangesFile( language, "a" );

    if  ( ! f )
	{ return -1;	}

    fprintf( f, "%s %s\n", prefix, word );

    fclose( f );

    return 0;
    }

-   (void)spellServer:		(NXSpellServer *)	sender
    didLearnWord:		(const char *)		word
    inLanguage:			(const char *)		language
    {
    int				accepted;
    const unsigned char *	s= word;
    unsigned char *		r= spelCopy;

    while( *s )
	{ *(r++)= NextToIso[*(s++)]; }
    *r= *s;

    /*
    DEB(printf( "LEARN  WORD '%s' IN LANGUAGE '%s'\n", word, language ));
    */

    if  ( indGet( &accepted, forgotInd, spelCopy ) >= 0	&&
	  accepted					)
	{ (void) indForget( forgotInd, spelCopy ); }

    if  ( indGet( &accepted, learntInd, spelCopy ) < 0	||
	  ! accepted					)
	{
	(void) indPut( learntInd, spelCopy );
	(void) toPrivateDict( "L", word, language );
	}

    return;
    }

-   (void)spellServer:		(NXSpellServer *)	sender
    didForgetWord:		(const char *)		word
    inLanguage:			(const char *)		language
    {
    int				accepted;
    const unsigned char *	s= word;
    unsigned char *		r= spelCopy;

    while( *s )
	{ *(r++)= NextToIso[*(s++)]; }
    *r= *s;

    /*
    DEB(printf( "FORGET WORD '%s' IN LANGUAGE '%s'\n", word, language ));
    */

    if  ( indGet( &accepted, learntInd, spelCopy ) >= 0	&&
	  accepted					)
	{ (void) indForget( learntInd, spelCopy ); }

    if  ( indGet( &accepted, forgotInd, spelCopy ) < 0	||
	  ! accepted					)
	{
	(void) indPut( forgotInd, spelCopy );
	(void) toPrivateDict( "F", word, language );
	}

    return;
    }

@end

extern int	spelMain(	const char *			argv0,
				const char *			language,
				const char *			vendor,
				const GuessSubstitution *	typos,
				int				typoCount )
    {
    NXSpellServer *	nxServer= [ [NXSpellServer alloc] init];

    if  ( [nxServer registerLanguage:	language byVendor: vendor ] )
	{
	[ nxServer setDelegate:
		[ [ indSpeller	alloc ]		initArgv0:	argv0
						language:	language
						typos:		typos
						typoCount:	typoCount ] ];
	[ nxServer run ];
	fprintf( stderr, "AFTER '[ nxServer run ]'\n" );
	}
    else{
	fprintf( stderr, "ELSE [nxServer registerLanguage:byVendor:\n" );
	}

    return 0;
    }

/************************************************************************/
/*  FAKE getc() on NeXT streams.					*/
/************************************************************************/
static int nxSpelFgetc( buf, textStream, pptr, pcnt )
char *		buf;
id		textStream;
char **		pptr;
int *		pcnt;
    {
    int	gotten;

    if  ( *pcnt > 0 )
	{ (*pcnt)--; return *((*pptr)++); }

    gotten= [ textStream	readCharacters:	buf
				count:		BUL	];
    if  ( gotten > 0 )
	{
	*pptr= buf+ 1; *pcnt= gotten- 1;
	return (int)*(unsigned char *)buf;
	}

    return EOF;
    }

/************************************************************************/
/*  Move a word from forgotten to learnt, or the other way round.	*/
/************************************************************************/
static int	spelMoveWord(	void *		fromInd,
				void *		toInd,
				unsigned char *	word )
    {
    int				rval= 0;
    int				accepted;
    const unsigned char *	s= word;
    unsigned char *		r= spelCopy;

    while( *s )
	{ *(r++)= NextToIso[*(s++)]; }
    *r= *s;

    if  ( indGet( &accepted, fromInd, spelCopy ) >= 0	&&
	  accepted					)
	{
	if  ( indForget( fromInd, spelCopy ) )
	    { rval= -1;	}
	}

    if  ( indGet( &accepted, toInd, spelCopy ) < 0	||
	  ! accepted					)
	{
	if  ( indPut( toInd, spelCopy ) < 0 )
	    { rval= -1;	}
	}

    return rval;
    }
