;; Enhancement to kill-all-local-variables
;; Copyright (C) 1988, 1991 Free Software Foundation, Inc.

;; This file is not officially part of GNU Emacs, but is being donated
;; to the Free Software Foundation.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY.  No author or distributor
;; accepts responsibility to anyone for the consequences of using it
;; or for whether it serves any particular purpose or works at all,
;; unless he says so in writing.  Refer to the GNU Emacs General Public
;; License for full details.

;; Everyone is granted permission to copy, modify and redistribute
;; GNU Emacs, but only under the conditions described in the
;; GNU Emacs General Public License.   A copy of this license is
;; supposed to have been given to you along with GNU Emacs so you
;; can know your rights and responsibilities.  It should be in a
;; file named COPYING.  Among other things, the copyright notice
;; and this notice must be preserved on all copies.

;; Created by: Joe Wells, jbw@bucsf
;; Created on: 1988?
;; Modified on: Thu Aug  8 10:01:15 1991
;; Filename: kill-fix.el
;; Purpose: make kill-all-local-variables skip certain variables
;;
;; Last modified by: Inge Frick, inge@nada.kth.se
;; Last modified on: Sun Jan  3 15:52:42 1993
;; Added the 'exit-function' property. One purpous of this is to allow
;; minor modes to exit gracefully when their local variables are killed.
;; When kill-all-local-variables kills a local variable that has an
;; exit-function property then that property must be a function of one argument
;; that will be called with the alist of all old local variables as argument.

;; save the original subr function definition of kill-all-local-variables
(or (fboundp 'original-kill-all-local-variables)
    (fset 'original-kill-all-local-variables
	  (symbol-function 'kill-all-local-variables)))

(defun kill-all-local-variables ()
  "Eliminate all the buffer-local variable values of the current buffer.
This buffer will then see the default values of all variables.
NOTE: This function has been modified to ignore buffer-local variables
      whose 'preserved' property is non-nil and to call the 'exit-function'
      property for those varaiables where it is non-nil."
  (let* ((oldvars (buffer-local-variables)) (ovars oldvars) var)
    (original-kill-all-local-variables)
    (while oldvars
      (if (eq (setq var (car (car oldvars))) 0) nil ; skip over 18.56 bug
	(if (get var 'preserved)
	    (progn (make-local-variable var) (set var (cdr (car oldvars)))))
	(if (setq var (get var 'exit-function)) (funcall var ovars)))
      (setq oldvars (cdr oldvars)))))

(provide 'kill-fix)
