;;; @ min-ispl.el - Electric minor mode interface to ispell package.
;;;
;;; $Id: min-ispl.el,v 5.3 1993/03/18 08:45:48 amanda Exp $

(provide 'min-ispl)
(require 'min-map)

;;; @@ Desciption
;;;
;;; In auto-ispell-mode `space' or `return' automatically trigger
;;; `ispell-word' on previously typed word.

;;; @@ Author
;;;
;;; Avishai Yacoby
;;; 
;;; Adopted to use minor keymaps by Per Abrahamsen
;;;
;;; This file is not part of the GNU Emacs distribution (yet).
;;;
;;; This file is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY.  No author or distributor
;;; accepts responsibility to anyone for the consequences of using it
;;; or for whether it serves any particular purpose or works at all,
;;; unless he says so in writing.  Refer to the GNU Emacs General Public
;;; License for full details.
;;;
;;; Everyone is granted permission to copy, modify and redistribute
;;; this file, but only under the conditions described in the
;;; GNU Emacs General Public License.   A copy of this license is
;;; supposed to have been given to you along with GNU Emacs so you
;;; can know your rights and responsibilities.  It should be in a
;;; file named COPYING.  Among other things, the copyright notice
;;; and this notice must be preserved on all copies.

;;; @@ The Mode Line

(defvar auto-ispell-mode nil
  "True, iff we are in auto ispell mode.")

  (make-variable-buffer-local 'auto-ispell-mode)

(or (assq 'auto-ispell-mode minor-mode-alist)
    (setq minor-mode-alist
	  (cons '(auto-ispell-mode " Spell") minor-mode-alist)))

;;; @@ The Mode

(defun auto-ispell-mode (arg)
  "Toggle auto-ispell-mode.
With prefix arg, turn auto-ispell-mode on iff arg is positive.

In auto-ispell-mode, inserting a space or a new line applies ispell-word
on the previous word.

Pressing M-TAB activates ispell-complete-word."
  (interactive "P")
  (if (or (and (null arg) auto-ispell-mode)
	  (<= (prefix-numeric-value arg) 0))
      ;; Turn it off
      (if auto-ispell-mode
	  (unbind-minor-mode 'auto-ispell-mode))
    ;; Turn it on
    (if auto-ispell-mode
	()
      (minor-set-key " " 'auto-ispell-check 'auto-ispell-mode)
      (minor-set-key "\r" 'auto-ispell-check 'auto-ispell-mode)
      (minor-set-key "\e\t" 'ispell-complete-word 'auto-ispell-mode)
      (minor-set-variable 'auto-ispell-mode t 'auto-ispell-mode)))
  (set-buffer-modified-p (buffer-modified-p)))

(defun auto-ispell-check ()
  (interactive "*")
  (minor-call-shadow 'auto-ispell-mode (this-command-keys))
  (save-excursion
    (backward-word 1)
    (ispell-word)))

;;; @@ Emacs

(run-hooks 'after-min-ispl-hook)

;;; Local Variables:
;;; mode: emacs-lisp
;;; mode: outline-minor
;;; outline-regexp: ";;; @+\\|(......"
;;; End:

