;;; @ tex-misc.el - Miscellaneous utilility functions for TeX mode.
;;; 
;;; $Id: tex-misc.el,v 5.21 1993/03/18 08:57:02 amanda Exp $

(provide 'tex-misc)

;;; @@ Copyright
;;;
;;; Copyright 1991 Kresten Krab Thorup
;;; Copyright 1992,1993 Per Abrahamsen
;;; 
;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 1, or (at your option)
;;; any later version.
;;; 
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;; 
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

;;; @@ Syntax Table

(defvar TeX-mode-syntax-table (make-syntax-table)
  "Syntax table used while in TeX mode.")

 (make-variable-buffer-local 'TeX-mode-syntax-table)

(progn ; Define TeX-mode-syntax-table.
  (modify-syntax-entry (string-to-char TeX-esc)
		           "\\" TeX-mode-syntax-table)
  (modify-syntax-entry ?\f ">"  TeX-mode-syntax-table)
  (modify-syntax-entry ?\n ">"  TeX-mode-syntax-table)
  (modify-syntax-entry ?$  "$$" TeX-mode-syntax-table)
  (modify-syntax-entry (string-to-char TeX-grop)
		           (concat "(" TeX-grcl)
		                TeX-mode-syntax-table)  
  (modify-syntax-entry (string-to-char TeX-grcl)
		           (concat ")" TeX-grop)
			        TeX-mode-syntax-table)  
  (modify-syntax-entry ?%  "<"  TeX-mode-syntax-table)
  (modify-syntax-entry ?\" "."  TeX-mode-syntax-table)
  (modify-syntax-entry ?&  "."  TeX-mode-syntax-table)
  (modify-syntax-entry ?_  "."  TeX-mode-syntax-table)
  (modify-syntax-entry ?@  "_"  TeX-mode-syntax-table)
  (modify-syntax-entry ?~  " "  TeX-mode-syntax-table)
  (modify-syntax-entry ?'  "w"  TeX-mode-syntax-table))

;;; @@ Keymap

(defvar TeX-mode-map nil
  "Keymap used in plain-TeX-mode.")

(if TeX-mode-map 
    ()
  (setq TeX-mode-map (make-sparse-keymap))

  ;; Standard
  (define-key TeX-mode-map "\177"     'backward-delete-char-untabify)
  (define-key TeX-mode-map "\C-c}"    'up-list)
  (define-key TeX-mode-map "\C-c#"    'normal-mode)
  (define-key TeX-mode-map "\C-c\C-n" 'normal-mode)
  (define-key TeX-mode-map "\C-c?"    'describe-mode)
  
  ;; From auc-tex.el
  (define-key TeX-mode-map "\""       'TeX-insert-quote)
  (define-key TeX-mode-map "\C-c{"    'TeX-insert-braces)
  (define-key TeX-mode-map "\C-c\C-f" 'TeX-font)
  (define-key TeX-mode-map "\C-c\C-m" 'TeX-insert-macro)

  ;; From tex-misc.el
  (define-key TeX-mode-map "\C-c;"    'TeX-comment-region)
  (define-key TeX-mode-map "\C-c%"    'TeX-comment-paragraph)

  (define-key TeX-mode-map "\C-c'"    'TeX-comment-out-paragraph) ;*** Old way
  (define-key TeX-mode-map "\C-c:"    'TeX-un-comment-region) ;*** Old way
  (define-key TeX-mode-map "\C-c\""   'TeX-un-comment) ;*** Old way

  ;; From tex-buf.el
  (define-key TeX-mode-map "\C-c\C-r" 'TeX-command-region)
  (define-key TeX-mode-map "\C-c\C-c" 'TeX-command-master)
  (define-key TeX-mode-map "\C-c!"    'TeX-command-master) ;Alias
  (define-key TeX-mode-map "\C-c\C-k" 'TeX-kill-job)
  (define-key TeX-mode-map "\C-c\C-l" 'TeX-recenter-output-buffer)
  (define-key TeX-mode-map "\C-c\C-h" 'TeX-home-buffer)

  ;; From tex-dbg.el
  (define-key TeX-mode-map "\C-c`"    'TeX-next-error)
  (define-key TeX-mode-map "\C-c\C-w" 'TeX-toggle-debug-boxes)
  (define-key TeX-mode-map "\C-c@"   'TeX-toggle-debug-boxes) ;*** Stupid alias

  ;; From tex.cpl.el
  (define-key TeX-mode-map "\C-c\t"   'TeX-complete-symbol))

;;; @@ Simple Comments

(defun TeX-comment-out-region (level start end)
  "Insert LEVEL comment characters before each line in the region."
  (interactive "*p\nr")
  (save-excursion
    (goto-char start)
    (beginning-of-line)
    (setq start (point))
    (goto-char end)
    (end-of-line)
    (setq end (point))
    (save-restriction 
      (narrow-to-region end start)
      (goto-char (point-min))
      (while (not (eobp))
	(let ((count level))
	  (while (> count 0)
	    (insert comment-start)
	    (setq count (- count 1))))
	(forward-line 1)))))

(defun TeX-un-comment-region (level start end)
  "Remove up to LEVEL comment characters from each line in the region."
  (interactive "*p\nr") 
  (save-excursion
    (goto-char start)
    (beginning-of-line)
    (setq start (point))
    (goto-char end)
    (end-of-line)
    (setq end (point))
    (save-restriction 
      (narrow-to-region start end)
      (goto-char (point-min))
      (while (not (eobp))
	(let ((count level))
	  (while (and (> count 0)
		      (not (eobp))
		      (looking-at (regexp-quote comment-start)))
	    (delete-char 1)
	    (setq count (- count 1))))
	(forward-line 1)))))

(defun TeX-un-comment (level)
  "Delete up to LEVEL %'s from the beginning of each line in a comment."
  (interactive "*p")
  (save-excursion
    ; Find first comment line
    (re-search-backward (concat "^[^" comment-start "]") nil 'limit)
    (let ((beg (point)))
      (forward-line 1)
      ; Find last comment line
      (re-search-forward (concat "^[^" comment-start "]") nil 'limit)
      ; Uncomment region
      (TeX-un-comment-region level beg (point)))))

(defun TeX-comment-out-paragraph (level)
  "Inserts LEVEL %'s at the beginning of every line in the current paragraph."
  (interactive "*p")
  (save-excursion
    (end-of-line)
    (start-of-paragraph-text)
    (beginning-of-line)
    (let ((beg (point)))
      (end-of-paragraph-text)
      ;; (forward-line -1)
      (beginning-of-line)
      (TeX-comment-out-region level beg (point)))))

;;; @@ Smart Comments

(defun TeX-comment-region (level begin end)
  "Remove or insert comments from region.

Without an argument, insert one level of comments in all lines in the
current region.

With a C-u argument, uncomment one level of comments from the
current region.

With a positive numeric argument, insert that many comment characters
in all lines in the current region.

With a negative numeric argument, remove that many comment character
from the current region."
  (interactive "*P\nr")
  (cond ((null level)
	 (TeX-comment-out-region 1 begin end))
	((listp level)
	 (TeX-un-comment-region 1 begin end))
	((eq level '-)
	 (TeX-un-comment-region 1 begin end))
	((> level 0)
	 (TeX-comment-out-region level begin end))
	((< level 0)
	 (TeX-un-comment-region (- level) begin end))))

(defun TeX-comment-paragraph (level)
  "Remove or insert comments from paragraph.

Without an argument, insert one level of comments in all lines in the
current paragraph.

With a C-u argument, uncomment one level of comments from the
surrounding comment lines.

With a positive numeric argument, insert that many comment characters
in all lines in the current paragraph.

With a negative numeric argument, remove that many comment character
from the surrounding comment lines."
  (interactive "*P")
  (cond ((null level)
	 (TeX-comment-out-paragraph 1))
	((listp level)
	 (TeX-un-comment 1))
	((eq level '-)
	 (TeX-un-comment 1))
	((> level 0)
	 (TeX-comment-out-paragraph level))
	((< level 0)
	 (TeX-un-comment (- level)))))

;;; @@ Indentation

(defun TeX-brace-count-line ()
  "Count number of open/closed braces."
  (if (looking-at "%")
      0
    (save-excursion
      (save-restriction
	(let ((count 0))
	  (narrow-to-region (point)
			    (save-excursion
			      (re-search-forward "[^\\\\]%\\|\n\\|\\'")
			      (backward-char)
			      (point)))

	  (if (looking-at "\\({\\|}\\)")
	    (if (string= "{" (TeX-match-buffer 1))
		(setq count (+ TeX-brace-indent-level count))
	      (setq count (- count TeX-brace-indent-level))))
	      
	  (while (re-search-forward "[^\\\\]\\({\\|}\\)" nil t)
	    (if (string= "{" (TeX-match-buffer 1))
		(setq count (+ TeX-brace-indent-level count))
	      (setq count (- count TeX-brace-indent-level)))
	    (if (looking-at "\\({\\|}\\)")
		(if (string= "{" (TeX-match-buffer 1))
		    (setq count (+ TeX-brace-indent-level count))
		  (setq count (- count TeX-brace-indent-level)))))
	  count)))))

(defvar TeX-brace-indent-level 2
  "*The level of indentation produced by a open brace.")

(defun TeX-comment-indent ()
  (if (looking-at "%%%")
      (current-column)
    (skip-chars-backward " \t")
    (max (if (bolp) 0 (1+ (current-column)))
	 comment-column)))

;;; @@ Fonts

(defvar TeX-font-list '((?\C-b "{\\bf " "}")
			(?\C-c "{\\sc " "}")
			(?\C-e "{\\em " "\\/}")
			(?\C-i "{\\it " "\\/}")
			(?\C-r "{\\rm " "}")
			(?\C-s "{\\sl " "\\/}")
			(?\C-t "{\\tt " "}"))
  "*List of fonts used by TeX-font.

Each entry is a list with three elements.  The first element is the
key to active the font.  The secont element is the string to insert
before point, and the third element is the string to insert after
point.")

(defun TeX-font (on-region)
  "Insert template for font change command.

If called with an argument, surround the current region with the
template."
  (interactive "*P")
  (let* ((char (read-char))
	 (entry (assoc char TeX-font-list)))
    (cond ((null entry)
	   (beep))
	  ((null on-region)
	   (insert (nth 1 entry))
	   (save-excursion
	     (insert (nth 2 entry))))
	  ((null (mark))
	   (error "Mark not set!"))
	  (t
	   ;; Put environment around region.
	   ;; A bit tricky, because we want point and mark to stay
	   ;; outside the font group (on the same side as before).
	   (let ((begin (min (point) (mark)))
		 (end (max (point) (mark))))
	     (if (< (point) (mark))
		 (save-excursion
		   (goto-char end)
		   (insert (nth 2 entry))
		   (goto-char begin)
		   (insert (nth 1 entry)))
	       (progn
		 (insert (nth 2 entry))
		 (save-excursion
		   (goto-char begin)
		   (insert (nth 1 entry))))))))))

;;; @@ Miscellaneous

(defun TeX-insert-quote (arg)
  "Insert ``, '' or \" according to preceding character.
With prefix argument, always insert \" characters."
  (interactive "*P")
  (if arg
      (let ((count (prefix-numeric-value arg)))
	(if (listp arg)
	    (self-insert-command 1)	;C-u always inserts just one
	  (self-insert-command count)))
    (insert (cond ((bobp) "``")
		  ((save-excursion
		     (forward-char -2)
		     (looking-at "``\\|''"))
		   (delete-backward-char 2)
		   ?\")
		  ((= (preceding-char) (string-to-char TeX-esc))
		   ?\")
		  ((save-excursion
		     (forward-char -1)
		     (looking-at "[ \t\n]\\|\\s("))
		   "``")
		  (t "''")))))

(defun TeX-insert-braces ()
  "Make a pair of braces and be poised to type inside of them."
  (interactive "*")
  (insert TeX-grop)
  (save-excursion
    (insert TeX-grcl)))

;;; @@ Emacs

(run-hooks 'TeX-after-tex-misc-hook)

;;; Local Variables:
;;; mode: emacs-lisp
;;; mode: outline-minor
;;; outline-regexp: ";;; @+\\|(......"
;;; End:

