.LP
.NH
Overriding Automatic Charset Handling
.LP
When the RTF Tools distribution is installed, the charset maps are installed
in
.I LIBDIR
(e.g.,
.I /usr/local/lib/rtf-tools ),
and the reader knows where to find them.
In the default case, all charset mapping decisions are made automatically by
the reader, including detection of the appropriate charset, reading the
charset map files, and switching to/from the Symbol font map as necessary.
If a translator wishes to override some of these choices itself, that can be
arranged, and is discussed in a later section.
.LP
There is some machinery in place in the reader at the moment for allowing
the writer to override some of this and do it itself, but there's no
external interface for accessing the machinery yet.
.LP
Translators can override certain aspects of the way the reader's
automatic charset handling:
.IP \(bu
Which files the reader reads automatically
.IP \(bu
Reading the charset files
.IP \(bu
Switching between the general and Symbol charset maps.
.LP
Actually, the following is at the moment hypothetical.
Most of the machinery for overriding the reader is in place internally,
but there's no external interface that the setup or writer code can access.
.NH 2
Reading Charset Maps
.LP
The maps to read are determined by the charset control word, which
precedes any document text in the RTF file.
The writer can catch the control word by installing a callback for the
.I rtfControl
token class and taking action when the callback sees a token with
a major number of
.I rtfCharSet .
The names of the charset maps
can be determined from the token text:
.LP
.DS
char file[rtfBufSiz];

sprintf (file, "%s\-gen", &rtfTextBuf[1]);
if (RTFReadCharset (file, rtfCSGeneral) == 0)
	RTFPanic ("Cannot read general charset map %s", file);
sprintf (file, "%s\-sym", &rtfTextBuf[1]);
if (RTFReadCharset (file, rtfCSSymbol) == 0)
	RTFPanic ("Cannot read symbol charset map %s", file);
.DE
.LP
The example shows how to read the default map files.
You can read whatever files you like.
.LP
If a file name is an absolute pathname,
.I RTFReadCharset()
tries to use it as is.
Otherwise,
.I RTFReadCharset()
tries
to find the file in the current directory, or, failing that, in
.I LIBDIR .
.LP
See
.I rtf2text
or
.I rtf2troff
for examples of how to catch and use the charset control.
.LP
If the RTF input file contains no charset control, the writer won't be able
to tell what maps to read in.
In this case, the reader assumes the ANSI charset and
.I ansi-gen
and
.I ansi-sym
are read in as necessary automatically.
.NH 2
Switching Between Charset Maps
.LP
To know when to switch charset maps, look at the font name
when the font changes in the input file.
For instance, in a switch statement processing character attribute controls,
you can do this:
.LP
.DS
RTFFont	*fp;

case rtfFontNum:
	if ((fp = RTFGetFont (rtfParam)) != (RTFFont *) NULL)
	{
		if (strncmp (fp->rtfFName, "Symbol", 6) == 0)
			RTFSetCharset (rtfCSSymbol);
		else
			RTFSetCharet (rtfCSGeneral)
	}
	break;

.DE
.LP
If your translator pushes and pops states when the group markers ``{''
and ``}'' are processed, it should set the charset when a state is popped,
since that may implicitly change the font.
.NH 2
Ignoring Symbol Font
.LP
If your translator wants to ignore the Symbol font, read in the general
map only, and don't ever switch the charset map.
(The default is the general map.)
