.\"
.\" $(TROFF) $(MSMACROS) file
.\"
.\" these macros are used in addition to the -ms macros
.so tmac.wrprc
.\"
.\" revision date - change whenever this file is edited
.ds RD 5 April 1994
.nr PO 1.25i	\" page offset 1.25 inches
.nr PD .5v	\" inter-paragraph distance
.\"
.EH 'RTF Processing Tool'- % -'Release 1.10'
.OH 'Release 1.10'- % -'RTF Processing Tool'
.OF 'Revision date:\0\0\*(RD''Printed:\0\0\n(dy \*(MO 19\n(yr'
.EF 'Revision date:\0\0\*(RD''Printed:\0\0\n(dy \*(MO 19\n(yr'
.\"
.\" IS/IE -- interface routine description header start/end.
.\" These set the indent, so the first IS must be preceded by
.\" .in +.25i and the description following the last .IE must be
.\" followed by .in -.25i.
.\"
.de IS	\" interface routine description header start
.in -.25i
.DS L
.ta .8i
.Cw
..
.de IE	\" interface routine description header end
.DE
.ft R
.in +.25i
..
.TL
.ps +2
A Tool for RTF Processing
.sp .5v
Release 1.10
.AU
Paul DuBois
dubois@primate.wisc.edu
.AI
Wisconsin Regional Primate Research Center
Revision date:\0\0\*(RD
.Ah Introduction
.LP
This document describes a general purpose tool for processing RTF
files\*-an RTF reader which may be configured in a
well-defined manner to allow it to be used with a variety of writers
generating different output formats.
This provides a method for generating RTF-to-\fIXXX\fR translators.
.LP
I assume that you have some familiarity with
RTF syntax and semantics, and that you're willing to study the source
code of the RTF distribution described here.
If you don't have the RTF specification, you can get it from the FTP site
listed under ``Distribution Availability'' at the end of this document.
References to ``the specification'' refer to the RTF specification document.
.LP
If you use this tool and find that you have an RTF file that won't pass
through the sample translator
.I rtf2null ,
or for which
.I rtf2null
announces unknown symbols, please contact me so the tool can
be improved.
It is best if you can supply the RTF file for which this behavior is observed.
.Ah "Theory of Operation"
.Bh "Translator Architecture"
.LP
This is a brief description of how translators are designed.
For more details, see the document
.I "RTF Tools Translator Architecture" .
.LP
There are three components to an RTF translator:
reader code, writer code, and driver code.
These break down as follows.
.IP reader
.br
Responsible for peeling tokens out of the input stream,
classifying them, and causing the writer to process them.
.IP writer
.br
Responsible for translating tokens from the input
stream into the required output format.
.IP driver
.br
Responsible for making sure the reader and writer are initialized, and
for calling the reader, to cause translation to occur.
.LP
This architecture allows the reader to remain constant, so that different
translators can be built by supplying different writer and driver code.
Also, for a given translator, the reader and writer remain constant and the
translator can be ported to different types of systems by supplying
system-specific driver code.
.LP
In practice, to build a new translator, you supply a
.I main()
function and the writer code, and link in the RTF reader.
.I main()
includes the driver code and is responsible to see that the following are done:
.Ls B
.Li
Determine which files are to be translated
.Li
Configure the reader, which may involve:
.Ls B "" \*-
.Li
Reset the input stream if necessary
.Li
Configure other reader behavior, such as whether or not to
process the font and color tables internally
.Li
Install writer callbacks into the reader so it knows what functions to
call when various kinds of tokens occur
.Le
.Li
Initialize the writer
.Li
Call the reader to process the input stream
.Li
Terminate the writer
.Le
A minimal translator (for a UNIX system) looks something like this:
.Ps
# include	<stdio.h>
# include	"rtf.h"

int
main ()
{
	RTFSetOpenLibFileProc (UnixOpenLibFile);
	RTFInit ();
	RTFRead ();
	exit (0);
}
.Pe
This installs a function that's suitable for opening RTF library
files on a UNIX system, initializes the reader, and calls it to read
.I stdin
(the default input stream).
The writer portion is null (i.e., there is no writer), so all that happens
is that the reader tokenizes the input and discards it.
That isn't very interesting;
most of the sample translators are examples of more elaborate translators.
.Bh "Reader Operation"
.LP
Each time a token is read, several global variables are set:
.LP
.DS
.ta 1.5i
\fIrtfClass\fP	token class
\fIrtfMajor\fP	token major number
\fIrtfMinor\fP	token minor number
\fIrtfParam\fP	token parameter value
\fIrtfTextBuf\fP	token text
\fIrtfTextLen\fP	length of token (including parameter text)
.DE
.LP
Tokens are classified using up to three numbers: token class, and major and
minor numbers.
The major and minor numbers may be meaningless depending on the kind
of token.
.LP
The class number can be:
.LP
.DS
.ta 1.5i
\fIrtfUnknown\fP	unrecognized token
\fIrtfGroup\fP	``{'' or ``}''
\fIrtfText\fP	plain text character
\fIrtfControl\fP	token beginning with ``\e''
\fIrtfEOF\fP	fake class number; indicates end of input stream
.DE
.LP
There are some exceptions.
A few tokens beginning with
.Cw \e
actually belong to other classes,
a tab character is treated like
.Cw \etab ,
and unrecognized tokens are put in class
.I rtfUnknown
no matter what they look like.
.LP
Within a class, tokens are assigned a major number, and perhaps a
minor number.
For the
.I rtfText
class, the major number is the value of the input character (0..255), and the
minor number is assigned a standard character code.
Text characters have different mappings in different RTF character sets, so to
avoid the problems associated with this, the reader maps the character onto
a standard character code using a charset-dependent translation table.
Translators should generally use the standard character code in
.I rtfMinor
rather than the raw character code in
.I rtfMajor .
Character mapping issues are described further in the document
.I "RTF Tools Character Mapping" .
.LP
A ``plain text'' character can be a literal character, a character specified
in hex notation
.Cw \e\' \fIxx\fP) (
or one of the special escaped characters
.Cw \e{ , (
.Cw \e} ,
.Cw \e\e ).
The sequence
.Cw \e:
is treated as a plain text colon.
This is arguably wrong; the rationale is given later under the
description of the
.I RTFGetToken()
function.
.LP
For the
.I rtfControl
class, most tokens have both a major and minor number.
For instance, all paragraph attribute control symbols have major number
.I rtfParAttr
and a minor number indicating a paragraph formatting property, such as
.I rtfLeftIndent
or
.I rtf\%SpaceBefore .
A few oddball control tokens have no minor number.
.LP
Control symbols may have a parameter value, e.g.,
.Cw \emargr720
specifies a right margin (in units of 720 twentieths of a point).
.LP
If no parameter value is given,
.I rtfParam
is
.I rtfNoParam .
.LP
Ideally, there should never be any tokens in the
.I rtfUnknown
class, but as the RTF standard continues to develop, unknown tokens are
inevitable.
.LP
To write a translator, you'll need to familiarize yourself with
the token classification scheme by reading
.I rtf.h .
A skeleton translator
.I rtfskel
is included with the distribution and may be used as a basis for new
translators.
.LP
As of release 1.10,
the reader allows an 8-bit character set since the current RTF specification
(version 1.2) now allows 8-bit characters.
Formerly, if the reader saw an 8-bit character, it converted the character
to the equivalent
.Cw \e\' \fIxx\fR
hex notation sequence and returned that as the token.
.LP
Generally, a translator will configure the RTF reader
to call particular writer
functions when certain kinds of tokens are encountered in the input
stream.
These functions are known as
.I "class callbacks" .
Writer callbacks can be registered with the reader using
.I RTFSetClassCallback()
for each token class.
.LP
The reader reads each token, classifies it, and sends it to a token routing
function
.I RTFRouteToken() ,
which tries to find a writer callback function to process the token.
Tokens in a given
class are ignored if no callback is registered for the class.
.LP
Class callbacks make it quite easy to receive notification when
certain types of tokens occur in the input.
For instance, a crude RTF text extractor could be written by
installing a callback function for the
.I rtfText
class.\**
.FS
Reasons this is a crude translator are that:
(i) some text characters occur in contexts where the characters
are not intended to be output, e.g., font tables, stylesheets; (ii)
some control symbols like
.Cw \etab
represent output text characters; (iii)
it writes output based on the raw input character value in
.I rtfMajor
rather than mapping the standard character code in
.I rtfMinor .
The sample translator
.I rtf2text
addresses these problems in a (slightly) more sophisticated manner.
.FE
Whenever the function is invoked,
.I rtfMajor
will contain a value in the range 0..255 representing the character value.
.Ps
# include	<stdio.h>
# include	"rtf.h"

void
TextCallback ()
{
	putchar (rtfMajor);
}
.Pe
.Ps
int
main ()
{
	RTFSetOpenLibFileProc (UnixOpenLibFile);
	RTFInit ();
	RTFSetClassCallback (rtfText, TextCallback);
	RTFRead ();
	exit (0);
}
.Pe
Callbacks for the
.I rtfControl
and
.I rtfGroup
classes
typically operate by selecting on the token major number to determine
the action to take.
A callback for the
.I rtfGroup
class usually will do something like this:
.Ps
void
BraceCallback ()
{
	switch (rtfMajor)
	{
	case rtfBeginGroup:
		\fI...push state...\fR
		break;
	case rtfEndGroup:
		\fI...pop state...\fR
		break;
	}
}
.Pe
.\"
.Bh "Destination Readers"
.\"
.LP
Grouping in RTF documents occurs within braces ``{'' and ``}''.
One kind of group is the
.I destination .
The token immediately following the opening brace is a destination
control symbol.
These indicate such things as headers, footers, footnotes, etc.
.LP
Three destinations which specify information for internal use (i.e.,
information which affects
output but isn't itself written) are the font table, color table and
stylesheet.
Since these three destinations occur so commonly and have a special syntax,
the RTF reader by default gobbles them up itself when it recognizes them.
The functions which do this are called
.I "destination readers"
and are probably the nearest thing in the reader to what might be
called parsers.
They are installed by default so that translators can be written
without the burden of understanding the syntax or digesting the
contents of these destinations.
Each of them constructs a list of the entries specified in the
destination and the reader includes functions providing access to
these lists.
.LP
Translators can turn off or override these defaults with
.I RTFSetDestinationCallback()
if necessary.
To override one, pass the address of a different destination reader
function.
To turn one off, pass
.Cw NULL .
.LP
Destination callbacks may be called for any destination, not just
.I rtfFontTbl ,
.I rtfColorTbl
and
.I rtfStyleSheet .
Destinations for which no callback is registered are not treated
specially.
.LP
Other destinations for which there is a default
reader are the information
.Cw \einfo ), (
picture
.Cw \epict ), (
and object
.Cw \eobject ) (
destinations; all they do is skip to the end of the group.
.Ch "Using the Built-in Destination Readers"
.LP
The font table, color table and stylesheet information is maintained
internally, and the reader either acts on that information itself, or
allows itself to be queried by the writer about it, as described
below.
These descriptions do not apply if the translator shuts off or
overrides the default destination readers, of course.
.LP
\fBStylesheet\*-\fRThe reader acts on this itself.
When the stylesheet destination is encountered, the style contents are
remembered.
Thereafter, whenever the writer receives notification that a style number
control symbol
.Cw \es \fInnn\fR) (
has occurred, it can call
.I RTFExpandStyle(rtfParam)
to cause the style to be expanded.
The reader consults contents of the stylesheet and each
token in the style definition is routed in turn back to the writer.
This effects a sort of macro expansion.
.LP
If the writer doesn't care about style expansion, it simply refrains
from calling
.I RTFExpandStyle() .
.LP
If the writer wants information about a style, it can call
.I RTFGetStyle() .
.LP
\fBFont table\*-\fRFor each entry in the font table, the font number,
type and name are maintained by the reader.
The writer finds out that a font number has been specified in the input
when its control class callback is invoked and
.I rtfMajor
\(eq
.I rtfCharAttr
and
.I rtfMinor
\(eq
.I rtfFontNum .
To obtain a pointer to the appropriate
.I RTFFont
structure, the reader function
.I RTFGetFont(rtfParam)
may be called.
.LP
\fBColor table\*-\fRFor each entry in the color table, the color number
is maintained along with the red, green and blue values.
The writer finds out that a color number has been specified in the input
when its control class callback is invoked and
.I rtfMajor
\(eq
.I rtfCharAttr
and
.I rtfMinor
\(eq
.I rtfColorNum .
To obtain a pointer to the appropriate
.I RTFColor
structure, the reader function
.I RTFGetColor(rtfParam)
may be called.
.LP
One subtle point about the built-in destination readers:
destinations cannot be recognized until
.I after
the occurrence of the ``{'' symbol that begins the destination.
This means the writer, if it maintains a state stack, will already
have pushed a state.
In order to allow the writer to properly pop that state in response
to the ``}'', these
destination readers feed the ``}'' back into the token router after
they pull it from the input stream.
What the writer actually sees is a ``{'' followed immediately by a
``}''.
.LP
Applications that maintain a state stack may find it necessary to do
something similar if they supply their own destination readers.
.Ah "Programming Interface"
.LP
Source files using the RTF reader should #include
.I rtf.h .
The library files common to all translators are used to build a library
.I librtf.a
in the distribution's
.I lib
directory.
This library should be part of the final application link.
.LP
The best way to learn how these source files work is to study the sample
translators, which vary in complexity from very simple (e.g.,
.I rtf2text ,
.I rtfwc ),
to wretchedly messy (e.g.,
.I rtf2troff ).
You should be aware that one implication of the way the translators
are built (callbacks and switch statements)
is that it's quite easy to build them incrementally.
You can start with a very bare-bones model, and start plugging in
callbacks as you progress.
Within the callbacks, your switch statements can progressively handle
more cases.
.LP
An alternative approach is to start with a copy of
.I rtfskel ,
which includes a full set of class callbacks and complete switch statements
for all tokens.
Each case is empty; you simply add code for those cases you want to handle.
You can also rip out the code for the cases you don't care about.
.Bh Types
.LP
Most types are pretty standard.
The one of note is
.I RTFFuncPtr ,
a generic function pointer which is defined like so:
.Ps
typedef	void (*RTFFuncPtr) ();
.Pe
That is, it's
a pointer to a function that takes no arguments and returns no value.
.\"
.Bh "Global variables"
.\"
.LP
The global RTF reader variables are:
.LP
.DS
.ta .6i 2i
int	rtfClass;	token class
int	rtfMajor;	token major number
int	rtfMinor;	token minor number
int	rtfParam;	parameter value for control symbols
char	*rtfTextBuf;	token text
int	rtfTextLen;	length of token text
.DE
.LP
These variables always apply to the token with which the writer should
be concerned.
This may be either the last token read or the current token within a
style which is being reprocessed.
.LP
.B Warning:
.I rtfTextBuf
is
.Cw NULL
until
.I RTFInit()
has been called.
.LP
Two other global variables which may be of interest provide the current
input line number and position within the line:
.LP
.DS
.ta .6i 2i
long	rtfLineNum;	current input line
int	rtfLinePos;	position within current line
.DE
.LP
These variables can be used to provide feedback to the user when a problem is
found in an input file as to the location of the problem.
They indicate the position immediately after the last token read.
.\"
.Bh Functions
.\"
.in +.25i
.IS
void
RTFInit ()
.IE
Initialize the RTF reader.
This should be called once for each input file to be processed.
It performs some initialization such as computing hash values for the
token lookup table and installation of some built-in destination and
token class readers.
.sp .5v
.I RTFInit()
may be called multiple times.
Each invocation resets the reader's state completely, except that the input
stream is not disturbed.
.IS
void
RTFRead ()
.IE
.I RTFRead()
calls
.I RTFGetToken()
to tokenize the input stream
and
.I RTFRouteToken()
to process each token, until input is exhausted.
When
.I RTFRead()
returns, input has been completely read and the writer can perform any
cleanup or termination needed.
.sp .5v
If you want to read multiple files per invocation of your translator,
you should do the following for each file:
call
.I RTFInit() ,
install callbacks, etc., then call
.I RTFRead() .
.IS
void
RTFRouteToken ()
.IE
This routine decides what to do with the current token and routes it
to the correct place for processing.
Usually this is directly to the writer via a class callback.
The token is
.I not
passed to the writer (i.e., the class callback is bypassed) when it
is a destination token for which a reader callback is installed.
.sp .5v
By default, built-in readers are installed
for font table, color table, stylesheet and information and picture
group destinations.
The built-in readers can be disabled
if the writer wants to see all tokens directly.
.IS
int
RTFGetToken ()
.IE
Reads one token from the input stream, classifies it, sets the global
variables, and returns the class number.
If the class is
.I rtfEOF
the end of the input stream has been reached.
Newlines
.Cw \en ), (
carriage returns
.Cw \er ), (
and nulls
are silently discarded by
.I RTFGetToken() ,
as they have no meaning.
All are passed to the token hook if one is installed, however.
.sp .5v
The sequence
.Cw \e:
is treated as a plain text character, with
.I rtfClass
set to
.I rtfText
and
.I rtfMajor
set to the colon ASCII code.
Strictly speaking,
.Cw \e:
is the control word for an index
subentry, but some versions of Microsoft Word write out plain text
colons with a preceding backslash, while others don't.
This unfortunate ambiguity results in an ugly dilemma.
It seems the lesser burden to require translators to recognize
that plain text colons should ``really'' be treated as index subentry
indicators
while inside of an index entry destination, than to recognize that an
index subentry control word should ``really'' be treated as a plain
text colon everywhere else.
.sp .5v
Writer code usually does not call
.I RTFGetToken()
directly except within specialized destination readers.
Driver code usually does not call
.I RTFGetToken()
if it calls
.I RTFRead() .
However, the following loop is an alternative to
.I RTFRead() :
.Ps
while (RTFGetToken () != rtfEOF)
{
	RTFRouteToken ();
}
.Pe
If a driver wants to regain control after reading each token, this loop
may be preferable to
.I RTFRead() .
.IS
int
RTFUngetToken ()
.IE
Pushes the last token back on the input stream so that
.I RTFGetToken()
returns it again.
You can't put back the same token twice unless you read it again
in the interim.
.IS
int
RTFPeekToken ()
.IE
Reads a token from the input stream and sets the global token variables,
but does not remove the token from the input stream.
.IS
void
RTFSetToken (class, major, minor, param, text)
int	class, major, minor, param;
char	*text;
.IE
It is sometimes useful to construct a fake token and run it through the
token router to cause the effects of the token to be applied.
.I RTFSetToken()
allows you to do this, by setting the reader's global variables to the
values supplied.
If
.I param
is
.I rtfNoParam ,
the token text
.I rtfTextBuf
is constructed from
.I text
and
.I param ,
otherwise
.I rtfTextBuf
is just copied from
.I text .
.IS
void
RTFSetReadHook (f)
RTFFuncPtr	f;
.IE
Install a function to be called by
.I RTFGetToken()
after each token is read from the input stream.
The function takes no arguments and returns no value.
Within the function,
information about the current token can be obtained from the global
variables.
This function is for token examination purposes only, and should not
modify those variables.
.IS
RTFFuncPtr
RTFGetReadHook ()
.IE
Returns a pointer to the current read hook, or
.Cw NULL
if there isn't one.
.IS
void
RTFSkipGroup ()
.IE
This function can be called to skip to the end of the current group (including
any subgroups).
It's useful for explicitly ignoring
.Cw \e*\e \fIdest\fP
groups, where
.I dest
is an unrecognized destination, or for causing groups that you don't want
to deal with to effectively ``disappear'' from the input stream.
.sp .5v
Calling this function in the middle of expanding a style may cause problems.
However, it is typically called when you have just seen a destination symbol,
which won't happen during a style expansion\*-I think.
.sp .5v
Be careful with this function if your writer maintains a state stack,
because you will already have pushed a state when the opening group
brace was seen.
After
.I RTFSkipGroup()
returns, the group closing brace has been read, and you'll need to pop
a state.
All global token variables will still be set to the closing brace, so
you may only need to call
.I RTFRouteToken()
to cause the state to be unstacked.
.IS
void
RTFExpandStyle (num)
int	num;
.IE
Performs style expansion of the given style number, or does nothing
if there is no such style.
The writer should call this when it notices that the current token
is a style number indicator.
.IS
void
RTFSetStream (stream)
FILE	*stream;
.IE
Redirects the RTF reader to the given stream.
This should be called before any reading is done.
The default input stream is
.I stdin .
An alternative to
.I RTFSetStream()
is to simply
.I freopen()
the input file on
.I stdin
(that's what all the sample translators do).
.sp .5v
The input stream is
.I not
modified by
.I RTFInit() .
.IS
void
RTFSetClassCallback (class, callback)
int		class;
RTFFuncPtr	callback;
.IE
Installs a writer callback function for the given token class.
The first argument is a class number, the second is the function to
call when tokens from that class are encountered in the input stream.
This will cause
.I RTFRouteToken()
to invoke the callback when it encounters a token in the class.
If
.I callback
is
.Cw NULL
(which is the default for all classes),
tokens in the class are ignored, i.e., discarded.
.sp .5v
The callback should take no arguments and return no value.
Within the callback,
information about the current token can be obtained from the global
variables.
.sp .5v
Installing a callback for the
.I rtfEOF
``class'' is silly and has no effect.
.IS
RTFFuncPtr
RTFGetClassCallback (class)
int	class;
.IE
Returns a pointer to the callback function for the given token class,
or
.Cw NULL
if there isn't one.
.IS
void
RTFSetDestinationCallback (dest, callback)
int		dest;
RTFFuncPtr	callback;
.IE
Installs a callback function for the given destination
.I dest "" (
is a token minor number).
When
.I RTFRouteToken()
sees a token with class
.I rtfControl
and major number
.I rtfDestination ,
it checks whether there is a callback for the destination indicated by
the minor number.
If so, it invokes it.
If
.I callback
is
.Cw NULL ,
the given destination is not treated specially (the control
class callback is invoked as usual).
By default, destination callbacks are installed for the font table, color
table, stylesheet, and information and picture group.
.sp .5v
The callback should take no arguments and return no value.
When the functon is invoked, the current token will be the destination
token following the destination's initial opening brace
.Cw { .
(For optional destinations, the destination token follows the
.Cw \e*
symbol.)
.IS
RTFFuncPtr
RTFGetDestinationCallback (dest)
int	dest;
.IE
Returns a pointer to the callback function for the given token class,
or
.Cw NULL
if there isn't one.
.IS
RTFStyle *
RTFGetStyle (num)
int	num;
.IE
Returns a pointer to the
.I RTFStyle
structure for the given style number.
The ``Normal'' style number is 0.
Pass \-1 to get a pointer to the first style in the list.
Styles are not stored in any particular order.
.sp .5v
Be sure to check the result; it might be
.Cw NULL .
.sp .5v
This function is meaningless if the default stylesheet destination
reader is overridden.
.IS
RTFFont *
RTFGetFont (num)
int	num;
.IE
Returns a pointer to the
.I RTFFont
structure for the given font number.
Pass \-1 to get a pointer to the first font in the list.
Fonts are not stored in any particular order.
.sp .5v
Be sure to check the result; it might be
.Cw NULL .
In particular, you might think that passing the number specified with
the
.Cw \edeff
(default font) control symbol would always yield a
valid font structure, but that's not true.
The default font might not be listed in the font table.
.sp .5v
This function is meaningless if the default font table destination
reader is overridden.
.IS
RTFColor *
RTFGetColor (num)
int	num;
.IE
Returns a pointer to the
.I RTFColor
structure for the given color number.
Pass \-1 to get a pointer to the first color in the list.
Colors are not stored in any particular order.
If the color values in the entry are \-1, the default color should be used.
The default color is translator-dependent.
.sp .5v
Be sure to check the result; it might be
.Cw NULL .
I think this means you should use the default color.
.sp .5v
This function is meaningless if the default color table destination
reader is overridden.
.IS
int
RTFCheckCM (class, major)
int	class, major;
.IE
Returns non-zero if
.I rtfClass
and
.I rtfMajor
are equal to
.I class
and
.I major ,
respectively, zero otherwise.
.IS
int
RTFCheckCMM (class, major, minor)
int	class, major, minor;
.IE
Returns non-zero if
.I rtfClass ,
.I rtfMajor
and
.I rtfMinor
are equal to
.I class ,
.I major
and
.I minor ,
respectively, zero otherwise.
.IS
int
RTFCheckMM (major, minor)
int	major, minor;
.IE
Returns non-zero if
.I rtfMajor
and
.I rtfMinor
are equal to
.I major
and
.I minor ,
respectively, zero otherwise.
.IS
char *
RTFAlloc (size)
int	size;
.IE
Returns a pointer to a block of memory
.I size
bytes long, or
.Cw NULL
if insufficient memory was available.
.IS
char *
RTFStrSave (s)
char	*s;
.IE
Allocates a block of memory big enough for a copy of the given string
(including terminating null byte), copies the string into it, and returns
a pointer to the copy.
Returns
.Cw NULL
if insufficient memory was available.
.IS
void
RTFFree (p)
char	*p;
.IE
Frees the block of memory pointed to by
.I p ,
which should have been allocated by
.I RTFAlloc()
or
.I RTFStrSave() .
It is safe to pass
.Cw NULL
to this routine.
.IS
void
RTFCharToHex (c)
char	c;
.IE
Returns 0..15 for the characters `0`..`9`,`a`..`f`.
.IS
void
RTFHexToChar (i)
int	i;
.IE
Returns the characters `0`..`9`,`a`..`f` for 0..15.
.IS
int
RTFReadCharSetMap (file, csId)
char	*file;
int	csId;
.IE
Reads a charset map file into the charset map indicated by
.I csId ,
which should be either
.I rtfCSGeneral
or
.I rtfCSSymbol .
Returns non-zero for success, zero otherwise.
.IS
void
RTFSetCharSetMap (file, csId)
char	*file;
int	csId;
.IE
Specify the name of the file to be read for the charset map indicated by
.I csId
(which should be either
.I rtfCSGeneral
or
.I rtfCSSymbol )
when auto-charset-file reading is done.
This can be used to override the default charset map names.
.I RTFSetCharSetMap()
should be called after
.I RTFInit()
but before you begin reading any input.
.IS
void
RTFSetCharSet (csId)
int	csId;
.IE
Switches to the charset map given by
.I csId ,
which should be either
.I rtfCSGeneral
or
.I rtfCSSymbol .
.IS
int
RTFGetCharSet ()
.IE
Returns the id of the current charset map, either
.I rtfCSGeneral
or
.I rtfCSSymbol .
.IS
int
RTFMapChar (c)
int	c;
.IE
Maps in input character onto a standard character code.
.IS
int
RTFStdCharCode (name)
char	*name;
.IE
Given a standard character name, returns the standard code corresponding to
the name, or \-1 if the name is unknown.
.IS
char *
RTFStdCharName (code)
int	code;
.IE
Given a standard character code, returns a string pointing to the standard
character name, or
.Cw NULL
if the code is unknown.
.IS
int
RTFReadOutputMap (file, outMap, reinit)
char	*file;
char	*outMap[];
int	reinit;
.IE
Reads an output map from the named file into
.I outMap .
If
.I reinit
is non-zero, the map is cleared first.
See the document
.I "RTF Tools Character Mapping"
for further details.
.sp .5v
Generally, the output map needs to be read only once.
.IS
void
RTFSetInputName (name)
char	*name;
.IE
.IS
void
RTFSetOutputName (name)
char	*name;
.IE
These functions tell the RTF library the input or output file names.
They're called by driver code so that writer code can determine the names
by calling
.I RTFGetInputName()
and
.I RTFGetOutputName() .
Since
.I RTFInit()
sets the names to
.Cw NULL ,
the driver should set the names after calling
.I RTFInit()
but before calling the writer to tell it to set up for a new file.
.IS
char *
RTFGetInputName ()
.IE
.IS
char *
RTFGetOutputName ()
.IE
These functions return pointers to the current input and output file names,
assuming the driver has set them up.
The caller should make a copy of the strings returned if it wants to modify
them.
.IS
void
RTFMsg (args ...)
.IE
This function
generates a diagnostic message.
It takes
.I printf() -like
arguments.
.sp .5v
See the description of
.I RTFSetMsgProc() .
.IS
void
RTFPanic (args ...)
.IE
This function generates an error message and terminates the process.
It takes
.I printf() -like
arguments.
.sp .5v
See the description of
.I RTFSetPanicProc() .
.IS
FILE *
RTFOpenLibFile (name, mode)
char	*name;
char	*mode;
.IE
This function opens a library file and returns a
.Cw FILE
pointer to it,
or
.Cw NULL
if the file could not be opened.
.sp .5v
See the description of
.I RTFSetOpenLibFileProc() .
.IS
void
RTFSetMsgProc (proc)
void	(*proc) ();
.IE
This function installs a function for use by
.I RTFMsg() ;
see
.I "RTF Tools Translator Architecture"
for details.
.IS
void
RTFSetPanicProc (proc)
void	(*proc) ();
.IE
This function installs a function for use by
.I RTFPanic() ;
see
.I "RTF Tools Translator Architecture"
for details.
.IS
void
RTFSetOpenLibFileProc (proc)
FILE	*(*proc) ();
.IE
This function installs a function that the library will use to open
library files.
The driver must call this when it starts up or
.I RTFOpenLibFile()
will always return
.Cw NULL .
The function should take a library file basename and open mode, open
the file, and return the
.Cw FILE
pointer, or
.Cw NULL
if the file could not be found and opened.
.in -.25i
.\"
.Ah "Distribution Availability"
.\"
.LP
This software may be redistributed without restriction and used for
any purpose whatsoever.
.LP
The RTF Tools distribution is available for anonymous 
.I ftp
access on
.I ftp.primate.wisc.edu .
Look in the
.I /pub/RTF
directory.
Updates appear there as they become available.
.LP
A version of the RTF specification is available in this directory,
as a binhex'ed Word for Macintosh document and in RTF and PostScript formats.
.LP
The software and documentation may also be accessed using gopher by connecting
to
.I gopher.primate.wisc.edu
or using World Wide Web
by connecting to
.I www.primate.wisc.edu
using the URL
.I http://www.primate.wisc.edu/ .
In both cases, look under ``Primate Center Software Archives''.
.LP
If you do not have Internet access, send requests to
.I software@primate.wisc.edu .
Bug reports and questions should be sent to this address
as well.
.LP
If you use this software as the basis for a translater not included in
the current collection, please send me a description that indicates how
it may be obtained and I'll add the description to the archive site.
