/*
 * prefs.c - Preferences file management
 *
 * ReadPrefs() reads the entire contents of the preferences file into
 * a generic block of memory and lets the caller interpret it.  This allows
 * the same function to be used by all translators.
 *
 * WritePrefs() writes a generic block of memory.  Caller passes the address
 * and size of the prefs structure it uses.
 */

# include	<Folders.h>
# include	<Script.h>

# include	<stdio.h>

# include	"rtf.h"
# include	"rtf-mac.h"


# define	prefsFileType	'pref'


/*
 * Open the application's preferences folder with the given mode.
 * If opening for writing, try to create it if it doesn't exist.
 */

static Boolean
OpenPrefsFile (StringPtr prefsName, OSType creator, short mode, short *pVRefNum)
{
OSErr	err;
short	vRefNum;
long	dirID, junk;
Str255	dirName;
CInfoPBRec	pbBlock;
FSSpec	fss;

	err = FindFolder (kOnSystemDisk, kPreferencesFolderType,
					kDontCreateFolder, &vRefNum, &dirID);
	if (err != noErr)
		return (false);

	/*
	 * Try to find "RTF Tools" folder in the Preferences folder.  If it's
	 * not there and we're opening the prefs file for writing, try to create
	 * the folder.
	 */

	err = FSMakeFSSpec (vRefNum, dirID, rtfLibFolderName, &fss);
	if (err == fnfErr && mode == fsWrPerm)
	{
		err = FSpDirCreate (&fss, smSystemScript, &junk);
		if (err != noErr)
			return (false);
	}

	/* now find dir ID of "RTF Tools" folder inside prefs folder */

	PStrCpy (dirName, rtfLibFolderName);
	pbBlock.dirInfo.ioNamePtr = dirName;
	pbBlock.dirInfo.ioFDirIndex = 0;	/* look for file named in dirName */
	pbBlock.dirInfo.ioDrParID = dirID;
	pbBlock.dirInfo.ioVRefNum = vRefNum;
	pbBlock.dirInfo.ioDrDirID = pbBlock.dirInfo.ioDrParID;
	err = PBGetCatInfo (&pbBlock, false);
	if (err != noErr)	/* no "RTF Tools" folder, apparently */
		return (false);

	RTFMsg ("Tools folder vRefNum %d, dirID %ld\n", vRefNum, pbBlock.dirInfo.ioDrDirID);

	/*
	 * Now have the vRefNum and dirID of the "RTF Tools" folder.  Create
	 * the FSSpec for the prefs file inside it. A "file not found" error
	 * is okay here if opening for writing; the file may not exist yet,
	 * but the FSSpec should be okay, so try to create the file.
	 */

	err = FSMakeFSSpec (vRefNum, pbBlock.dirInfo.ioDrDirID, prefsName, &fss);
	if (err == fnfErr && mode == fsWrPerm)
		err = FSpCreate (&fss, creator, prefsFileType, smSystemScript);
	if (err == noErr)
		err = FSpOpenDF (&fss, mode, pVRefNum);
	return (err == noErr);
}


Boolean
ReadPrefs (StringPtr name, char **p, long *size)
{
Boolean	result = false;
short	vRefNum;
long	count;

	if (!OpenPrefsFile (name, 0L, fsRdPerm, &vRefNum))
		return (false);

	if (GetEOF (vRefNum, size) == noErr)
	{
		if ((*p = RTFAlloc ((int) *size)) != (Ptr) NULL)
		{
			count = *size;
			if (FSRead (vRefNum, &count, (Ptr) *p) == noErr)
				result = true;
			else
				RTFFree (*p);
		}
	}
	(void) FSClose (vRefNum);
	return (result);
}


Boolean
WritePrefs (StringPtr name, OSType creator, char *p, long size)
{
Boolean	result = false;
short	vRefNum;
long	count;

	if (!OpenPrefsFile (name, creator, fsWrPerm, &vRefNum))
		return (false);

	count = size;
	if (FSWrite (vRefNum, &count, (Ptr) p) == noErr && count == size)
		result = true;
	else
		size = 0;	/* clobber contents to prevent misinterpretation on next read */
	(void) SetEOF (vRefNum, size);
	(void) FSClose (vRefNum);
	return (result);
}
