/*
 * text-mac.c -- Macintosh rtf2text driver (for THINK C).
 */

# include	<stdio.h>

# include	"TransSkel.h"

# include	"rtf-mac.h"
# include	"rtf2text.h"


# define	normalHilite	0
# define	dimHilite		255


# define	optionsMenuRes	2000

# define	prefsDlogRes	2000


typedef enum
{
	iOK = 1,
	iCancel,
	iDlogTitle,
	iCreatorTitleStatText,
	iCreatorEditText,
	iSelectAppPushButton,
	iOutputSuffixStatText,
	iOutputSuffixEditText,
	iErrorSuffixStatText,
	iErrorSuffixEditText,
	iAutoOutputNameCheckBox,
	iAutoErrorNameCheckBox,
	iOKOutlineUserItem,
	iLineUserItem
};


static void PrefsDialog (void);


/*
 * Stuff to tell wrapper how to initialize/shutdown the writer, how to
 * begin/end processing for each output file, how to allow for
 * application-specific menu handling stuff, and default file extension
 */

static AppInfo	myAppInfo =
{
	nil,				/* no menu init */
	nil,				/* no menu adjust */

	WriterInit,
	nil,				/* no writer final cleanup needed */

	BeginFile,
	nil,				/* no per-file cleanup needed */

	PrefsDialog,		/* preferences dialog function */

	"\prtf",			/* input file suffix */
	"\ptxt",			/* output file suffix */
	"\perr",			/* error file suffix */
	
	0L,					/* output file creator */
	0L,					/* error file creator */

	/* flags */
	persistentStatus
	| prefsInEditMenu
	| usesOutputFile | autoNameOutputFile | autoOpenOutputFile
	| usesErrorFile | autoNameErrorFile
};


/* ------------------------------------------------------------------- */

/*
 * Preferences dialog routines
 */


static void
OSTypeToStr (OSType type, StringPtr str)
{
StringPtr	typeStr;

	typeStr = (StringPtr) &type;
	str[0] = 4;
	BlockMove (typeStr, &str[1], (long) 4);
}


static void
StrToOSType (StringPtr str, OSType *type)
{
short	i;

	for (i = 1; i <= 4; i++)
	{
		if (i <= str[0])
			((StringPtr) type)[i-1] = str[i];
		else
			((StringPtr) type)[i-1] = ' ';	/* blank fill on right */
	}
}


static void
SetOSType (DialogPtr dlog, short item, OSType type)
{
Str255		str;

	OSTypeToStr (type, str);
	SkelSetDlogStr (dlog, item, str);
	SelIText (dlog, item, 0, 32767);
}


static void
GetOSType (DialogPtr dlog, short item, OSType *type)
{
Str255		str;

	SkelGetDlogStr (dlog, item, str);
	StrToOSType (str, type);
}


static void
SelectApplication (DialogPtr dlog, short item)
{
FSSpec	fssApp;
FInfo	fndrInfo;

	if (SelectInputFile (&fssApp, 'APPL'))
	{
		if (FSpGetFInfo (&fssApp, &fndrInfo) != noErr)
			SysBeep (1);
		else
			SetOSType (dlog, item, fndrInfo.fdCreator);
	}
}


static pascal void
DrawUserLine (DialogPtr dlog, short item)
{
Rect	r;

	SkelGetDlogRect (dlog, item, &r);
	FrameRect (&r);
}


static void
PrefsDialog (void)
{
ModalFilterProcPtr	filter;
DialogPtr	dlog;
GrafPtr		savePort;
Str255		str, str2;
short		item;
OSType		creatorType;
short		hilite;
Rect		r;
short		loop = 1;

	dlog = GetNewDialog (prefsDlogRes, nil, (WindowPtr) -1L);
	if (dlog == (DialogPtr) nil)
	{
		SysBeep (1);
		return;
	}

	SkelPositionWindow (dlog, skelPositionOnParentDevice,
									FixRatio (1, 2), FixRatio (1, 5));

	/* install outliner for default item */

	SkelSetDlogButtonOutliner (dlog, iOKOutlineUserItem);

	creatorType = myAppInfo.outputCreator;
	if (creatorType == 0L)
		creatorType = docCreator;		/* default creator */
	SetOSType (dlog, iCreatorEditText, creatorType);

	SkelSetDlogStr (dlog, iOutputSuffixEditText, myAppInfo.outputSuffix);
	SkelSetDlogStr (dlog, iErrorSuffixEditText, myAppInfo.errorSuffix);

	SkelSetDlogCtlValue (dlog, iAutoOutputNameCheckBox,
							TstAppFlag (&myAppInfo, autoNameOutputFile));
	SkelSetDlogCtlValue (dlog, iAutoErrorNameCheckBox,
							TstAppFlag (&myAppInfo, autoNameErrorFile));

	SkelSetDlogProc (dlog, iLineUserItem, DrawUserLine);

	GetPort (&savePort);
	SetPort (dlog);

	ShowWindow (dlog);
	InitCursor ();
	
	while (loop)
	{
		filter = SkelDlogFilter (nil, true);
		SkelDlogDefaultItem (iOK);
		SkelDlogCancelItem (iCancel);
		ModalDialog (filter, &item);
		SkelRmveDlogFilter ();
		switch (item)
		{
		case iCancel:
		case iOK:
			loop = 0;
			break;
		case iSelectAppPushButton:
			SelectApplication (dlog, iCreatorEditText);
			break;
		case iAutoOutputNameCheckBox:
		case iAutoErrorNameCheckBox:
			(void) SkelToggleDlogCtlValue (dlog, item);
			break;
		}

		/*
		 * Check text in creator field and hilite OK button according to
		 * whether or not it's empty.  If the button changes state, invalidate
		 * the surrounding rectangle so the outline gets redrawn, too.
		 * Do the same for the suffix fields, which can't be empty, either.
		 */

		hilite = normalHilite;
		SkelGetDlogStr (dlog, iCreatorEditText, str);
		if (str[0] == 0)
			hilite = dimHilite;
		SkelGetDlogStr (dlog, iOutputSuffixEditText, str);
		if (str[0] == 0)
			hilite = dimHilite;
		SkelGetDlogStr (dlog, iErrorSuffixEditText, str);
		if (str[0] == 0)
			hilite = dimHilite;
		if (SkelSetDlogCtlHilite (dlog, iOK, hilite))
		{
			SkelGetDlogRect (dlog, iOKOutlineUserItem, &r);
			InvalRect (&r);
		}
	}

	SetPort (savePort);
	if (item == iOK)
	{
		GetOSType (dlog, iCreatorEditText, (OSType *) &myAppInfo.outputCreator);
		SkelGetDlogStr (dlog, iOutputSuffixEditText, str);
		if (str[0] > 31)
			str[0] = 31;
		PStrCpy (myAppInfo.outputSuffix, str);
		SkelGetDlogStr (dlog, iErrorSuffixEditText, str);
		if (str[0] > 31)
			str[0] = 31;
		PStrCpy (myAppInfo.errorSuffix, str);

		if (SkelGetDlogCtlValue (dlog, iAutoOutputNameCheckBox))
			myAppInfo.flags |= autoNameOutputFile;
		else
			myAppInfo.flags &= ~autoNameOutputFile;
		if (SkelGetDlogCtlValue (dlog, iAutoErrorNameCheckBox))
			myAppInfo.flags |= autoNameErrorFile;
		else
			myAppInfo.flags &= ~autoNameErrorFile;
	}
	DisposeDialog (dlog);
}


/* ------------------------------------------------------------------- */


int
main (void)
{
	MacintoshWrapper (&myAppInfo);
}
